@@ -27,6 +27,7 @@ TBC: How much time does this take. Everything is translated ??
 from .source.AmoebaFitter import AmoebaFitter
 from .source.AnnealingAmoeba import AnnealingAmoeba
 from .source.ArctanModel import ArctanModel
+from .source.AstropyModel import AstropyModel
 from .source.BasicSplinesModel import BasicSplinesModel
 from .source.BSplinesModel import BSplinesModel
 from .source.BaseFitter import BaseFitter
@@ -34,9 +35,6 @@ from .source.BaseModel import BaseModel
 from .source.BernoulliErrorDistribution import BernoulliErrorDistribution
 from .source.BirthEngine import BirthEngine
 from .source.BracketModel import BracketModel
-#from .source.CategoricalFitter import CategoricalFitter
-#from .source.CategoricalLikelihood import CategoricalLikelihood
-#from .source.CategoricalProblem import CategoricalProblem
 from .source.CauchyErrorDistribution import CauchyErrorDistribution
 from .source.CauchyPrior import CauchyPrior
 from .source.CircularUniformPrior import CircularUniformPrior
@@ -46,9 +44,9 @@ from .source.ClassicProblem import ClassicProblem
 from .source.CombiModel import CombiModel
 from .source.ConstantModel import ConstantModel
 from .source.ConvergenceError import ConvergenceError
-#from .source.CrossEngine import CrossEngine
 from .source.CurveFitter import CurveFitter
-#from .source.CyclicResults import CyclicResults
+#from .source.DarwinModel import DarwinModel
+#from .source.DarwinProblem import DarwinProblem
 from .source.DeathEngine import DeathEngine
 from .source.DecisionTreeModel import DecisionTreeModel
 from .source.Dynamic import Dynamic
@@ -64,12 +62,17 @@ from .source.ExponentialErrorDistribution import ExponentialErrorDistribution
 from .source.ExponentialPrior import ExponentialPrior
 from .source.Fitter import Fitter
 from .source.FixedModel import FixedModel
+from .source.FootballModel import FootballModel
 #from .source.FreeShape2dModel import FreeShape2dModel
 from .source.FreeShapeModel import FreeShapeModel
 from .source.GalileanEngine import GalileanEngine
 from .source.GaussErrorDistribution import GaussErrorDistribution
+from .source.Gauss2dErrorDistribution import Gauss2dErrorDistribution
 from .source.GaussModel import GaussModel
 from .source.GaussPrior import GaussPrior
+#from .source.Gene import *
+#from .source.Genome import Genome
+#from .source.GenomeMaker import GenomeMaker
 from .source.GibbsEngine import GibbsEngine
 from .source.HarmonicModel import HarmonicModel
 from .source.HarmonicDynamicModel import HarmonicDynamicModel
@@ -97,11 +100,12 @@ from .source.MonteCarlo import MonteCarlo
 from .source.MultipleOutputProblem import MultipleOutputProblem
 from .source.NestedSampler import NestedSampler
 #from .source.NeuralNetModel import NeuralNetModel
+from .source import NeuralNetUtilities
 from .source.NoiseScale import NoiseScale
 from .source.NonLinearModel import NonLinearModel
-#from .source.OrderEngine import OrderEngine
 from .source.OrthonormalBasis import OrthonormalBasis
 from .source.PadeModel import PadeModel
+from .source.PhantomCollection import PhantomCollection
 from .source.PhantomSampler import PhantomSampler
 from .source.PoissonErrorDistribution import PoissonErrorDistribution
 from .source.PolySineAmpModel import PolySineAmpModel
@@ -141,23 +145,25 @@ from .source.StructureEngine import StructureEngine
 from .source.SurfaceSplinesModel import SurfaceSplinesModel
 from .source.UniformErrorDistribution import UniformErrorDistribution
 from .source.UniformPrior import UniformPrior
+from .source.UserModel import UserModel
 from .source.VoigtModel import VoigtModel
 from .source.Walker import Walker
 from .source.WalkerList import WalkerList
 
 ## OrderProblems
-#from .source.DistanceCostFunction import DistanceCostFunction
-#from .source.NestedSolver import NestedSolver
-#from .source.LoopEngine import LoopEngine
-#from .source.MoveEngine import MoveEngine
-#from .source.OrderEngine import OrderEngine
-#from .source.OrderProblem import OrderProblem
-#from .source.ReverseEngine import ReverseEngine
-#from .source.SalesmanProblem import SalesmanProblem
-#from .source.ShuffleEngine import ShuffleEngine
+from .source.DistanceCostFunction import DistanceCostFunction
+from .source.LoopEngine import LoopEngine
+from .source.MoveEngine import MoveEngine
+from .source.NearEngine import NearEngine
+from .source.NestedSolver import NestedSolver
+from .source.OrderEngine import OrderEngine
+from .source.OrderProblem import OrderProblem
+from .source.ReverseEngine import ReverseEngine
+from .source.SalesmanProblem import SalesmanProblem
+from .source.ShuffleEngine import ShuffleEngine
 #from .source.StartNearEngine import StartNearEngine
-#from .source.StartOrderEngine import StartOrderEngine
-#from .source.SwitchEngine import SwitchEngine
+from .source.StartOrderEngine import StartOrderEngine
+from .source.SwitchEngine import SwitchEngine
 
 from .source.Formatter import formatter
 from .source.Formatter import fma
