@@ -5,9 +5,9 @@ from .Engine import Engine
 from .Formatter import formatter as fmt
 
 __author__ = "Do Kester"
-__year__ = 2021
+__year__ = 2024
 __license__ = "GPL"
-__version__ = "2.7.0"
+__version__ = "3.2.1"
 __url__ = "https://www.bayesicfitting.nl"
 __status__ = "Perpetual Beta"
 
@@ -30,7 +30,7 @@ __status__ = "Perpetual Beta"
 #  * Science System (HCSS), also under GPL3.
 #  *
 #  *    2003 - 2014 Do Kester, SRON (Java code)
-#  *    2017 - 2021 Do Kester
+#  *    2017 - 2024 Do Kester
 
 class BirthEngine( Engine ):
     """
@@ -41,12 +41,16 @@ class BirthEngine( Engine ):
 
     The member is kept when the logLikelihood > lowLhood.
 
+    Attributes from Engine
+    ----------------------
+    walkers, errdis, maxtrials, slow, rng, report, phantoms, verbose
+
     Author       Do Kester.
 
     """
 
     #  *********CONSTRUCTORS***************************************************
-    def __init__( self, walkers, errdis, slow=None, copy=None, seed=23455, verbose=0 ) :
+    def __init__( self, walkers, errdis, copy=None, **kwargs ) :
         """
         Constructor.
 
@@ -56,16 +60,13 @@ class BirthEngine( Engine ):
             walkers to be diffused
         errdis : ErrorDistribution
             error distribution to be used
-        slow : None or int > 0
-            Run this engine every slow-th iteration. None for all.
         copy : BirthEngine
             to be copied
-        seed : int
-            for random number generator
+        kwargs : for Engine
+            "phantoms", "slow", "seed", "verbose"
 
         """
-        super( ).__init__( walkers, errdis, slow=slow, copy=copy,
-                    seed=seed, verbose=verbose )
+        super( ).__init__( walkers, errdis, copy=copy, **kwargs ) 
 
     def copy( self ):
         """ Return copy of this.  """
@@ -107,7 +108,7 @@ class BirthEngine( Engine ):
         ftry = walker.fitIndex
 
         if self.verbose > 4 :
-            print( "BEN0  ", walker.id, walker.parent, fmt( allp, max=None ), len( ftry ) )
+            print( "Birth  ", walker.id, walker.parent, fmt( allp, max=None ), len( ftry ) )
 
         off = 0
         model = problem.model
@@ -121,11 +122,11 @@ class BirthEngine( Engine ):
         np = model.npbase
 
         if self.verbose > 4 :
-            print( "BEN1  ", walker.id, nc, np, len( allp ), len( ftry ) )
+            print( "       ", walker.id, nc, np, len( allp ), len( ftry ) )
 
         if not ( nc < model.growPrior.unit2Domain( self.rng.rand() ) and
                  model.grow( offset=off, rng=self.rng ) ):
-            self.reportReject()
+            self.reportFailed()
             return 0
 
         dnp = model.npbase - np         # parameter change
