@@ -10,9 +10,9 @@ from .Formatter import formatter as fmt
 from .Formatter import fma
 
 __author__ = "Do Kester"
-__year__ = 2021
+__year__ = 2024
 __license__ = "GPL3"
-__version__ = "2.7.0"
+__version__ = "3.2.1"
 __url__ = "https://www.bayesicfitting.nl"
 __status__ = "Perpetual Beta"
 
@@ -35,7 +35,7 @@ __status__ = "Perpetual Beta"
 #  * Science System (HCSS), also under GPL3.
 #  *
 #  *    2010 - 2014 Do Kester, SRON (Java code)
-#  *    2017 - 2021 Do Kester
+#  *    2017 - 2024 Do Kester
 
 class ChordEngine( Engine ):
     """
@@ -62,14 +62,12 @@ class ChordEngine( Engine ):
 
     Attributes
     ----------
-    nstep : int
-        average number of orthogonal steps
     debug : bool
         perform the step-out action too
 
     Attributes from Engine
     ----------------------
-    walkers, errdis, slow, maxtrials, rng, verbose, report, unitRange, unitMin
+    walkers, errdis, slow, maxtrials, nstep, rng, verbose, report, unitRange, unitMin
 
     Author       Do Kester.
 
@@ -92,7 +90,6 @@ class ChordEngine( Engine ):
         """
         super( ).__init__( walkers, errdis, copy=copy, **kwargs )
 
-        self.nstep = 5
         self.maxtrials = 25
         self.plotter = DummyPlotter()
         self.debug = False
@@ -136,18 +133,16 @@ class ChordEngine( Engine ):
         param = walker.allpars
         usav = self.domain2Unit( problem, param[fitIndex], kpar=fitIndex )
         self.plotter.start( param=param )
+#        self.startJourney( usav )
 
-        # Determine n-dim box boundaries in which the parameters are located
-        if np > len( self.unitRange ) :
-            self.unitRange = numpy.ones( np, dtype=float )
-            self.unitMin = numpy.zeros( np, dtype=float )
+        # Determine n-dim box boundaries in which the parameters are located 
+        uran, umin = self.getUnitRange( problem, lowLhood )
+        uran = uran[fitIndex]
+        umin = umin[fitIndex]
 
-        uran = self.unitRange[fitIndex]
-        umin = self.unitMin[fitIndex]
-
-        dur = 2 * numpy.max( uran ) / len( self.walkers )       # extend range a bit
+        dur = 5 * numpy.max( uran ) / len( self.walkers )       # extend range a bit
         uran += 2 * dur
-        umin = self.unitMin[fitIndex] - dur
+        umin -= dur
         umax = umin + uran
 
         # make sure that 0 <= umin < usav < umax <= 1
@@ -164,11 +159,11 @@ class ChordEngine( Engine ):
         vel = self.rng.rand( np ) - 0.5
 
         if self.verbose > 4 :
-            print( "++++++++++++++++++++++++++++++++++++++++++" )
-            print( walker.id, nstep, fmt(lowLhood), fmt( param ) )
-            print( "umin  ", fmt( umin ) )
-            print( "umax  ", fmt( umax ) )
-            print( "usav  ", fmt( usav ) )
+            print( "Chord     LogL  ", fmt( walker.logL ), " LowL  ", fmt( lowLhood), nstep, self.maxtrials )
+            print( "alpar ", fma( param, linelength=200 ) )
+            print( "usav  ", fma( usav ) )
+            print( "umin  ", fma( umin ) )
+            print( "umax  ", fma( umax ) )
 
         reset = True
         ptry = param.copy()
@@ -179,13 +174,14 @@ class ChordEngine( Engine ):
             ## orthonormalise the random vector
             vel = onb.normalise( vel, reset=reset )
 
-            if self.verbose > 4 :
-                print( ks, step, "vel   ", fmt( vel ) )
-
             ## append travel times wrt usav (==0) to all edges umin and umax
             ## tt contains entrance times as negative values
             ## and exit times as positive values.
             tt = numpy.append( ( umin - usav ) / vel, ( umax - usav ) / vel )
+            if self.verbose > 4 :
+                print( "uvel  ", fma( vel ) )
+                print( "ttb   ", fma( tt[:10] ) )
+                print( "tte   ", fma( tt[10:] ) )
 
             ## find smallest entrance and exit times
             t1 = numpy.min( numpy.where( tt > 0, tt, +math.inf ) )
@@ -194,19 +190,33 @@ class ChordEngine( Engine ):
             ## same now for the values at the edges of the unit box (0,1)
             if self.debug :
                 tt = numpy.append( - usav / vel, ( 1.0 - usav ) / vel )
+                if self.verbose > 4 :
+                    print( "tt0   ", fma( tt[:10] ) )
+                    print( "tt1   ", fma( tt[10:] ) )
                 t1max = numpy.min( numpy.where( tt > 0, tt, +math.inf ) )
                 t0max = numpy.max( numpy.where( tt < 0, tt, -math.inf ) )
 
-                t0 = self.stepOut( problem, ptry, usav, vel, t0, t0max, lowLhood, fitIndex )
-                t1 = self.stepOut( problem, ptry, usav, vel, t1, t1max, lowLhood, fitIndex )
+                tt0 = self.stepOut( problem, ptry, usav, vel, t0, t0max, lowLhood, fitIndex )
+                tt1 = self.stepOut( problem, ptry, usav, vel, t1, t1max, lowLhood, fitIndex )
+
+                print( fmt( ( t0max, tt0, t0, t1, tt1, t1max ), max=None ) )
+                t0 = tt0
+                t1 = tt1
+
+            if self.verbose > 4 :
+                print( ks, step, "vel   ", fmt( vel ), fmt( tt ), fmt( t0 ), fmt( t1 ) )
 
             kk = 0
             while True :
                 kk += 1
 
-                assert t0 < 0 < t1, "%f < %f < %f"%(t0,0,t1)
+                # assert t0 < 0 < t1, "%f < %f < %f"%(t0,0,t1)
+                if not ( t0 < 0 < t1 and math.isfinite( t0 ) and math.isfinite( t1 ) ) : 
+                    break
+
                 ## dt is timestep wrt. usav (at 0)
                 dt = t0 + self.rng.rand( 1 ) * ( t1 - t0 )
+
                 utry = usav + vel * dt
 
                 ptry[fitIndex] = self.unit2Domain( problem, utry, kpar=fitIndex  )
@@ -222,13 +232,14 @@ class ChordEngine( Engine ):
                     step += 1
 
                     self.plotter.move( param, ptry, col=0, sym=0 )
+#                    self.calcJourney( usav - utry )
 
                     ## check if better than Lbest in walkers[-1]
                     # self.checkBest( problem, ptry, Ltry, fitIndex )
 
                     ## keep the trial parameters
                     param = ptry.copy()
-                    usav = self.domain2Unit( problem, param[fitIndex], kpar=fitIndex )
+                    usav = utry
 
                     # make sure that 0 <= umin < usav < umax <= 1
                     umin = numpy.where( usav < umin, 2 * usav - umin, umin )
@@ -270,7 +281,7 @@ class ChordEngine( Engine ):
 
         self.plotter.stop( param=param, name="ChordEngine" )
 
-        return step
+        return step * np            # nr of successfull parameter moves
 
 
     def stepOut( self, problem, ptry, usav, vel, t, tmax, lowLhood, fitIndex ) :
@@ -301,7 +312,3 @@ class ChordEngine( Engine ):
                     return tmax
 
         return t
-
-
-
-
