@@ -1,12 +1,15 @@
 import numpy as numpy
 from . import Tools
 
+from .LevenbergMarquardtFitter import LevenbergMarquardtFitter
 from .Walker import Walker
+from .PhantomCollection import PhantomCollection
+from .Formatter import formatter as fmt
 
 __author__ = "Do Kester"
-__year__ = 2021
+__year__ = 2024
 __license__ = "GPL3"
-__version__ = "2.7.2"
+__version__ = "3.2.1"
 __url__ = "https://www.bayesicfitting.nl"
 __status__ = "Perpetual Beta"
 
@@ -29,7 +32,7 @@ __status__ = "Perpetual Beta"
 #  * Science System (HCSS), also under GPL3.
 #  *
 #  *    2010 - 2014 Do Kester, SRON (Java code)
-#  *    2017 - 2021 Do Kester
+#  *    2017 - 2024 Do Kester
 
 class Engine( object ):
     """
@@ -46,6 +49,8 @@ class Engine( object ):
         error distribution to be used
     slow : int
         If slow > 0, run this engine every slow-th iteration.
+    phantoms : PhantomCollection
+        Collection of valid walker positions collected during engine execution
     maxtrials : int
         maximum number of trials for various operations
     rng : numpy.random.RandomState
@@ -63,6 +68,9 @@ class Engine( object ):
     Author       Do Kester.
 
     """
+    NSTEP  = 5              #  number of steps
+    MAXTRIALS = 5           # maximum number of trials
+
     SUCCESS = 0             #  succesfull move
     REJECT = 1              #  rejected move
     FAILED = 2              #  failed to move
@@ -71,10 +79,13 @@ class Engine( object ):
 
     #  *********CONSTRUCTORS***************************************************
 
-    def __init__( self, walkers, errdis, slow=None, copy=None, seed=4213, verbose=0 ):
+    def __init__( self, walkers, errdis, slow=None, phantoms=None, copy=None, 
+                    seed=4213, verbose=0 ):
         """
         Constructor.
 
+        Only one PhantomCollection should be present for all Engines.
+
         Parameters
         ----------
         walkers : list of Walker
@@ -83,43 +94,63 @@ class Engine( object ):
             error distribution to be used
         slow : None or int > 0
             Run this engine every slow-th iteration. None for all.
+        phantoms : None or PhantomCollection
+            Container for all valid walkers, that have been tried. But were not kept.
+            To calculate the spread of the parameters vs likelihood.
         seed : int
             for random number generator
         verbose : int
             report about the engines when verbose > 4
         copy : Engine
-            engine to be copied
-
+            engine to be copied 
         """
         self.walkers = walkers
         self.errdis = errdis
         self.report = [0]*5
 
+        self.checkBest = self.noBoost
+ 
         if copy is None :
-            self.maxtrials = 5
+            self.maxtrials = self.MAXTRIALS
+            self.nstep = self.NSTEP
             self.rng = numpy.random.RandomState( seed )
-            self.unitRange = None
-            self.unitMin = None
             self.verbose = verbose
             if slow is not None : self.slow = slow
-            # self.setWalker = self.setWalkerAdd2List if usePhantoms else self.setWalkerInPlace
-            # self.lastWalkerId = 0
+            if not phantoms is None :
+                self.phantoms = phantoms
         else :
             self.maxtrials = copy.maxtrials
+            self.nstep = copy.nstep
             self.rng = copy.rng
-            self.unitRange = copy.unitRange
-            self.unitMin   = copy.unitMin
             self.verbose   = copy.verbose
             if hasattr( copy, "slow" ) : self.slow = copy.slow
-            # self.setWalker = copy.setWalker
-            # self.lastWalkerId = copy.lastWalkerId
+            self.phantoms = copy.phantoms
 
     def copy( self ):
         """ Return a copy of this engine.  """
         return Engine( self.walkers, self.errdis, copy=self )
 
+    def bestBoost( self, problem, myFitter=None ) :
+        """
+        When a logL is found better that all the rest, try to update
+        it using a fitter.
+
+        Parameters
+        problem : Problem
+            the problem at hand
+        myFitter : None or Fitter
+            None fetches LevenbergMarquardtFitter
+            a (non-linear) fitter
+        """
+        self.checkBest = self.doBoost
+        if myFitter is None :
+            myFitter = LevenbergMarquardtFitter
+        self.fitter = myFitter( problem.xdata, problem.model )
+
+
+
     #  *********SET & GET***************************************************
-    def setWalker( self, kw, problem, allpars, logL, fitIndex=None ) :
+    def setWalker( self, kw, problem, allpars, logL, walker=None, fitIndex=None ) :
         """
         Update the walker with problem, allpars, LogL and logW.
 
@@ -136,37 +167,71 @@ class Engine( object ):
             list of all parameters
         logL : float
             log Likelihood
+        walker : Walker or None
+            Copy this walker or create new one
         fitIndex : array_like
             (new) fitIndex
         """
-        id = 0 if kw >= len( self.walkers ) else self.walkers[kw].id
-        walker = Walker( id, problem, allpars, fitIndex )
-        walker.logL = logL
-        self.walkers.setWalker( walker, kw )
+        if walker is None :
+            id = 0 if kw >= len( self.walkers ) else self.walkers[kw].id
+            wlkr = Walker( id, problem, allpars, fitIndex )
+        else :
+            wlkr = walker.copy()
+            wlkr.allpars = allpars    # Tools.toArray( allpars, ndim=1, dtype=float )
 
-#       DONT DO THIS ANY MORE        
-#        self.checkBest( problem, allpars, logL, fitIndex=fitIndex )
+        wlkr.logL = logL
+        self.checkBest( wlkr )
 
+        self.walkers.setWalker( wlkr, kw )
 
-    def checkBest( self, problem, allpars, logL, fitIndex=None ) :
+        self.phantoms.storeItems( wlkr )
+
+    def noBoost( self, walker ) :
+        pass
+
+    def doBoost( self, walker ) :
         """
-        Check if Ltry better than the best at self.walkers[-1].
-        If so replace.
+        Check if walker is best in phantoms and try to optimize.
 
         Parameters
         ----------
-        problem : Problem
-            the problem in the walker
-        logL : float
-            likelihood
-        allpars : array_like
-            parameters of problem
-        fitIndex : array_like
-            (new) fitIndex
+        walker : Walker
+            new walker to be checked
         """
-        if logL > self.walkers[-1].logL :
-            self.setWalker( -1, problem, allpars.copy(), logL, fitIndex )
-            self.reportBest()
+        wlist = self.phantoms.getList( walker )
+
+        if wlist is None or wlist[-1].logL >= walker.logL :
+            return
+
+#        print( "Best   ", fmt( walker.logL ) )
+#        print( fmt( walker.allpars, max=None ) )
+
+        problem = walker.problem
+        nh = self.errdis.nphypar
+        par0 = walker.allpars[:-nh]
+        try :
+            self.fitter.model = problem.model
+            pars = self.fitter.fit( problem.ydata, par0=par0 )
+        except :
+            return
+
+        if nh == 0 :
+            ptry = pars
+        elif walker.fitIndex[-1] == -1 :
+            ptry = numpy.append( pars, self.fitter.getScale() )
+        else :
+            ptry = numpy.append( pars, walker.allpars[-1] )
+
+        Ltry = self.errdis.logLikelihood( problem, ptry )
+
+        if Ltry > walker.logL :
+            walker.allpars = ptry
+            walker.logL = Ltry
+
+#        print( "Better ", fmt( walker.logL ) )
+#        print( fmt( walker.allpars, max=None ) )
+
+        self.reportBest()
 
 ######## domain <> unit ###########################################
 
@@ -231,6 +296,35 @@ class Engine( object ):
                 dval[i] = self.errdis.unit2Domain( uval[i], kp )
         return dval
 
+    def startJourney( self, unitStart ) :
+        """
+        Calculate the starting position and reset
+
+        Parameters
+        ----------
+        unitStart : array_like
+            start position in npars-dimensions in unit space
+        """
+        self.journey = 0
+        self.jstart = numpy.sqrt( numpy.sum( unitStart * unitStart ) )
+
+    def calcJourney( self, unitDistance ) :
+        """
+        Calculate the distance travelled since reset
+
+        Parameters
+        ----------
+        unitDistance : array_like
+            step size in npars-dimensions in unit space
+        """
+        self.journey += numpy.sqrt( numpy.sum( unitDistance * unitDistance ) )
+
+    def reportJourney( self ) :
+        try :
+            return ( self.jstart, self.journey )
+        except :
+            return ( 0,0 )
+
     def makeIndex( self, np, val ) :
         kpar = [k for k in range( np )]
         nh = len( val ) - np
@@ -265,62 +359,77 @@ class Engine( object ):
         """
         self.report[self.BEST] += 1
 
-    def printReport( self ) :
-        print( " %10d %10d %10d %10d %10d" % (self.report[0], self.report[1],
+    def printReport( self, best=False ) :
+        if best :
+            print( " %10d %10d %10d %10d %10d" % (self.report[0], self.report[1],
                               self.report[2], self.report[3], self.report[4] ) )
-
-
-    def calculateUnitRange( self ):
-        """
-        Calculate the range of the present parameter values in unit values.
-
-        For Dynamic models the range is calculated for those parameters present in all models;
-        it is 1.0 for other parameters.
-
-        """
-        kmx = 0
-        if not self.walkers[0].problem.model.isDynamic() :
-            npmax = len( self.walkers[0].allpars )
         else :
-            npmax = 0
-            for k, walker in enumerate( self.walkers ) :
-                if len( walker.allpars ) > npmax :
-                    npmax = len( walker.allpars )
-                    kmx = k
+            print( " %10d %10d %10d %10d" % (self.report[0], self.report[1],
+                              self.report[2], self.report[4] ) )
 
-        minv = self.walkers[kmx].allpars.copy()
-        maxv = self.walkers[kmx].allpars.copy()
-        nval = numpy.zeros( npmax, dtype=int )
+    def successRate( self ) :
+        """
+        Return percentage of success.
+        """
+        if not hasattr( self, "save" ) :
+            self.save = [0, 0]
 
-        for walker in self.walkers :
-            fi = walker.fitIndex
-#            print( "Eng   ", fi, minv, walker.allpars )
-            minv[fi] = numpy.fmin( minv[fi], walker.allpars[fi] )
-            maxv[fi] = numpy.fmax( maxv[fi], walker.allpars[fi] )
-            nval[fi] += 1
+        srate = ( 100 * ( self.report[0] - self.save[0] ) / 
+                  ( self.report[0] + self.report[1] + self.report[2] - self.save[1] ) )
+        self.save = [self.report[0], ( self.report[0] + self.report[1] + self.report[2] )]
+        return srate
 
-        problem = self.walkers[kmx].problem
-        fi = self.walkers[kmx].fitIndex
 
-        maxv[fi] = self.domain2Unit( problem, maxv[fi], kpar=fi )
-        minv[fi] = self.domain2Unit( problem, minv[fi], kpar=fi )
+    def getUnitMinmax( self, problem, lowLhood ) :
+        """
+        Calculate unit minimum and maximum from the Phantoms
 
-        self.unitRange = numpy.abs( maxv - minv )
-        self.unitMin = numpy.fmin( minv, maxv )
+        Parameters
+        ----------
+        problem : Problem
+            To extract the unit range for
+        lowLhood : float
+            low likelihood boundary
+        """
+        npars = problem.npars
+        pamin, pamax = self.phantoms.getParamMinmax( lowLhood, np=problem.npars )
 
-#        print( "Eng1  ", self.unitRange )
-        return
+        npars += self.errdis.nphypar
+        umax = ( numpy.ones( npars, dtype=float )  if pamax is None
+                else self.domain2Unit( problem, pamax ) )
+        umin = ( numpy.zeros( npars, dtype=float )  if pamin is None
+                else self.domain2Unit( problem, pamin ) )
+        umin = numpy.fmin( umin, umax )
+
+        return ( umin, umax )
+
+    def getUnitRange( self, problem, lowLhood ) :
+        """
+        Calculate unit range and minimum from PhantomCollection
+
+        Parameters
+        ----------
+        problem : Problem
+            To extract the unit range for
+        lowLhood : float
+            low likelihood boundary
+        """
+        umin, umax = self.getUnitMinmax( problem, lowLhood )
+ 
+        uran = numpy.abs( umax - umin )
+                
+        return ( uran, umin )
 
     def __str__( self ) :
         return str( "Engine" )
 
-    def execute( self, walker, lowLhood ):
+    def execute( self, kw, lowLhood ):
         """
-        Execute the engine for diffusing the parameters
+        Execute the engine for difusing the parameters
 
         Parameters
         ----------
-        walker : Sample
+        kw : walker-id
             walker to diffuse
         lowLhood : float
             low limit on the loglikelihood
@@ -335,10 +444,10 @@ class Engine( object ):
 
 class DummyPlotter( object ) :
 
-    def __init__( self, iter=0 ) :
+    def __init__( self, iter=1 ) :
         self.iter = iter
 
-    def start( self, param=None ):
+    def start( self, param=None, ulim=None ):
         """ start the plot. """
         pass
 
