@@ -4,11 +4,12 @@ import math
 from .HyperParameter import HyperParameter
 from . import Tools
 from .Tools import setAttribute as setatt
-
+from .Formatter import formatter as fmt
+ 
 __author__ = "Do Kester"
-__year__ = 2021
+__year__ = 2023
 __license__ = "GPL3"
-__version__ = "2.8.0"
+__version__ = "3.1.0"
 __url__ = "https://www.bayesicfitting.nl"
 __status__ = "Perpetual Beta"
 
@@ -31,7 +32,7 @@ __status__ = "Perpetual Beta"
 #  * Science System (HCSS), also under GPL3.
 #  *
 #  *    2010 - 2014 Do Kester, SRON (Java code)
-#  *    2017 - 2021 Do Kester
+#  *    2017 - 2023 Do Kester
 
 
 class ErrorDistribution( object ):
@@ -213,7 +214,7 @@ class ErrorDistribution( object ):
             param = problem.model.parameters
         else :
             nh = self.nphypar
-            param = allpars[:-nh]
+            param = allpars[:-nh] if nh > 0 else allpars
 
         res2 = problem.weightedResSq( param )
 
@@ -405,7 +406,8 @@ class ErrorDistribution( object ):
 
             return dL
         except :
-            print( "Using numeric partialLogL." )
+#            raise
+#            print( "Using numeric partialLogL." )
             return self.numPartialLogL( problem, allpars, fitIndex )
 
 
@@ -444,12 +446,14 @@ class ErrorDistribution( object ):
         dL = numpy.zeros( len( fitIndex ), dtype=float )
         p = allpars.copy( )
         i = 0
+
         for k in fitIndex :
             p[k] = allpars[k] - self.deltaP
             lm = self.logLikelihood( problem, p )
             p[k] = allpars[k] + self.deltaP
             lp = self.logLikelihood( problem, p )
             dL[i] = 0.5 * ( lp - lm ) / self.deltaP
+
             i += 1
             p[k]= allpars[k]
         return dL
