@@ -20,13 +20,16 @@ from .SampleList import SampleList
 
 from .Problem import Problem
 from .ClassicProblem import ClassicProblem
+from .LinearModel import LinearModel
 from .ErrorsInXandYProblem import ErrorsInXandYProblem
 from .MultipleOutputProblem import MultipleOutputProblem
 from .EvidenceProblem import EvidenceProblem
+from .PhantomCollection import PhantomCollection
 
 from .ErrorDistribution import ErrorDistribution
 from .ScaledErrorDistribution import ScaledErrorDistribution
 from .GaussErrorDistribution import GaussErrorDistribution
+from .Gauss2dErrorDistribution import Gauss2dErrorDistribution
 from .LaplaceErrorDistribution import LaplaceErrorDistribution
 from .PoissonErrorDistribution import PoissonErrorDistribution
 from .CauchyErrorDistribution import CauchyErrorDistribution
@@ -41,16 +44,19 @@ from .ChordEngine import ChordEngine
 from .GibbsEngine import GibbsEngine
 from .GalileanEngine import GalileanEngine
 from .StepEngine import StepEngine
+from .RandomEngine import RandomEngine
 ## for Dynamic Models import the classes
 from .BirthEngine import BirthEngine
 from .DeathEngine import DeathEngine
 ## for Modifiable Models:
 from .StructureEngine import StructureEngine
 
+from .BaseFitter import BaseFitter
+
 __author__ = "Do Kester"
-__year__ = 2021
+__year__ = 2024
 __license__ = "GPL3"
-__version__ = "2.8.1"
+__version__ = "3.2.1"
 __url__ = "https://www.bayesicfitting.nl"
 __status__ = "Perpetual Beta"
 
@@ -73,7 +79,7 @@ __status__ = "Perpetual Beta"
 #  * Science System (HCSS), also under GPL3.
 #  *
 #  *    2003 - 2014 Do Kester, SRON (Java code)
-#  *    2017 - 2021 Do Kester
+#  *    2017 - 2024 Do Kester
 
 class NestedSampler( object ):
     """
@@ -145,9 +151,6 @@ class NestedSampler( object ):
     4. StepEngine.
         It moves all parameters in a random direction. It is a randomwalk.
 
-    5. CrossEngine.
-        The parameters of 2 walkers are mixed in a random way.
-
     For dynamic models 2 extra engines are defined:
 
     6. BirthEngine.
@@ -194,6 +197,8 @@ class NestedSampler( object ):
         minimum number of iterations (adapt when starting problems occur)
     end : float (2.0)
         stopping criterion
+    tolerance : float (-12)
+        stopping criterion: stop if log( dZ / Z ) < tolerance
     verbose : int
         level of blabbering
     walkers : WalkerList
@@ -212,12 +217,16 @@ class NestedSampler( object ):
     """
     TWOP31 = 2 ** 31        ## 2 ** 32 raises an error on Windows systems.
     ENSEMBLE = 100
+    TOLERANCE = -12
+    END = 2
+    RATE = 1.0
 
     #  *********CONSTRUCTORS***************************************************
     def __init__( self, xdata=None, model=None, ydata=None, weights=None,
-                problem=None, distribution=None, limits=None, keep=None, ensemble=100,
-                discard=1, seed=80409, rate=1.0, engines=None, maxsize=None,
-                threads=False, verbose=1 ) :
+                accuracy=None, problem=None, distribution=None, limits=None, 
+                keep=None, ensemble=ENSEMBLE, discard=1, seed=80409, rate=RATE,
+                bestBoost=False,
+                engines=None, maxsize=None, threads=False, verbose=1 ) :
         """
         Create a new class, providing inputs and model.
 
@@ -235,6 +244,9 @@ class NestedSampler( object ):
             array of dependent (to be fitted) data
         weights : array_like (None)
             weights pertaining to ydata
+        accuracy : float or array_like
+            accuracy scale for the datapoints
+            all the same or one for each data point
         problem : None or string or Problem
             Defines the kind of problem to be solved.
 
@@ -274,10 +286,14 @@ class NestedSampler( object ):
             number of walkers
         discard : int
             number of walkers to be replaced each generation
-        seed : int
+        seed :int
             seed of random number generator
         rate : float
             speed of exploration
+        bestBoost : bool or Fitter
+            False   no updates of best logLikelihood
+            True    boost the fit using LevenbergMarquardtFitter
+            fitter  boost the fit using this fitter.
         engines : None or (list of) string or (list of) Engine
             to randomly move the walkers around, within the likelihood bound.
 
@@ -310,22 +326,13 @@ class NestedSampler( object ):
         if problem is None :
             problem = "classic"
 
-#        else :
-#            if model is None :
-#                model = problem.model
-#            if xdata is None :
-#                xdata = problem.xdata
-#            if ydata is None :
-#                ydata = problem.ydata
-#            if weights is None :
-#                weights = problem.weights
-
-        self.setProblem( problem, model=model, xdata=xdata, ydata=ydata, weights=weights )
+        self.setProblem( problem, model=model, xdata=xdata, ydata=ydata, weights=weights,
+                         accuracy=accuracy )
 
         if model is None :
             model = self.problem.model
 
-        if not model.hasPriors() and model.npars > 0:
+        if model is not None and not model.hasPriors() and model.npars > 0:
             warnings.warn( "Model needs priors and/or limits" )
 
         self.keep = keep
@@ -333,6 +340,10 @@ class NestedSampler( object ):
         if ensemble is not None :
             self.ensemble = ensemble
 
+        self.minimumIterations = 100            # to prvent premature stops
+        self.maxIterations = None               # only for testing
+        self.repiter = 100                      # report status every repiter
+        self.tolerance = self.TOLERANCE
         self.discard = discard
 
         self.rng = numpy.random.RandomState( seed )
@@ -342,8 +353,9 @@ class NestedSampler( object ):
         self.rate = rate
         self.restart = None                     ## TBD
 
-        self.minimumIterations = 100
-        self.end = 2.0
+        self.bestBoost = bestBoost
+
+        self.end = self.END
         self.maxtrials = 5
         self.threads = threads
 
@@ -352,16 +364,29 @@ class NestedSampler( object ):
         if distribution is not None :
             self.setErrorDistribution( distribution, limits=limits )
         elif limits is not None :
-            self.setErrorDistribution( self.problem.myDistribution(), limits=limits )
+            self.setErrorDistribution( limits=limits )
         else :
-            self.setErrorDistribution( self.problem.myDistribution(), scale=1.0 )
+            self.setErrorDistribution( scale=1.0 )
 
         self.setEngines( engines )
 
         ## Initialize the sample list
         self.samples = SampleList( model, 0, ndata=self.problem.ndata )
 
+        """
+        The order in which the main attributes are created.
 
+        where       method                  attribute
+        __init__    setProblem              self.problem
+        __init__    setErrorDistribution    self.distribution
+        __init__    setEngines              self.phantoms
+                                            self.engines
+        __init__                            self.samples
+        sample      initSample
+                      initWalkers           self.walkers
+                        setInitialEngine    self.initialEngine
+        """                                            
+    
     #  *******SAMPLE************************************************************
     def sample( self, keep=None, plot=False ):
         """
@@ -382,32 +407,48 @@ class NestedSampler( object ):
             "iter" 	show iterations
             "all"  	show iterations and final result
             "last" 	show final result
-
+            "test"      plot iterations but dont show (for testing)
         """
         keep = self.initSample( keep=keep )
 
-        tail = self.initReport( keep=keep, plot=plot )
+        if ( self.problem.hasAccuracy and self.walkers[0].fitIndex[-1] < 0 and 
+                not isinstance( self.distribution, GaussErrorDistribution ) ) :
+            raise AttributeError( "%s cannot be combined with accuracies and variable scale" %
+                                    self.distribution ) 
+
+#        if self.minimumIterations is None :
+#            self.minimumIterations = 10 * self.ensemble / self.discard
+
+        tail = self.initReport( keep=keep )
 
         explorer = Explorer( self, threads=self.threads )
 
+        ## move all walkers around for initial exploration of the complete space.
+        if not isinstance( self.problem.model, LinearModel ) :
+            self.lowLhood = -sys.float_info.max
+            # Explore all walker(s)
+            explorer.explore( range( self.ensemble ), self.lowLhood, self.iteration )
+            self.iteration = 0                  # reset iteration number
+
+
         self.logZ = -sys.float_info.max
+        self.logdZ = 0
         self.info = 0
         self.logWidth = math.log( 1.0 - math.exp( -1.0 / self.ensemble ) )
 
-
 #        TBD put self.logWidth in the saved file too
 #        if self.optionalRestart() :
 #            self.logWidth -= self.iteration * ( 1.0 * self.discard ) / self.ensemble
 
+        self.histinsert = []
         ## iterate until done
-        while self.iteration < self.getMaxIter( ):
+#        while self.iteration < self.getMaxIter( ):
+        while self.nextIteration() :
 
             self.walkers.sort( key=self.walkerLogL )    # sort the walker list on logL
 
-            self.calculateUnitRange()
-
             worst = self.worst                      # the worst are low in the sorted ensemble
-            self.lowLhood = self.walkers[worst].logL
+            self.lowLhood = self.walkers[worst-1].logL
 
             self.updateEvidence( worst )            # Update Z and H and store posterior samples
 
@@ -419,6 +460,9 @@ class NestedSampler( object ):
 
             self.updateWalkers( explorer, worst )
 
+            newL = self.walkers[worst-1].logL
+            self.histinsert += [self.walkers.firstIndex( newL )]
+
             self.optionalSave( )
 
 
@@ -431,7 +475,7 @@ class NestedSampler( object ):
         self.samples.normalize( )
 
         # put the info into the model
-        if self.problem.model and not self.problem.model.isDynamic() :
+        if self.problem.model and not self.problem.isDynamic() :
             self.problem.model.parameters = self.samples.parameters
             self.problem.model.stdevs = self.samples.stdevs
 
@@ -452,13 +496,16 @@ class NestedSampler( object ):
         for eng in self.engines :
             eng.walkers = self.walkers
             eng.lastWalkerId = len( self.walkers )
+            if self.bestBoost :
+                eng.bestBoost( self.problem, myFitter=self.myFitter )
 
         self.distribution.ncalls = 0                      #  reset number of calls
 
         return keep
 
-    def sortLogL( self, walkers ) :
-        return numpy.argsort( [w.logL for w in walkers] )
+#   Not in Use
+#    def sortLogL( self, walkers ) :
+#        return numpy.argsort( [w.logL for w in walkers] )
 
     def walkerLogL( self, w ) :
         return w.logL
@@ -481,6 +528,8 @@ class NestedSampler( object ):
         for sp in self.distribution.hyperpar :
             if not sp.isFixed and sp.isBound() :
                 fitlist += [nh]                             # it is to be optimised
+            elif self.problem.hasAccuracy and nh == -self.distribution.nphypar :
+                allpars[nh] = 0                             # fix (model)scale at 0
             else :
                 allpars[nh] = self.distribution.hypar[nh]   # fill in the fixed value
 
@@ -501,29 +550,39 @@ class NestedSampler( object ):
 
         return ( fitlist, allpars )
 
-    def calculateUnitRange( self ) :
-        self.engines[0].calculateUnitRange()
-        for eng in self.engines :
-            eng.unitRange = self.engines[0].unitRange
-            eng.unitMin   = self.engines[0].unitMin
-
 
     def doIterPlot( self, plot ) :
+        """
+        Returns
+        -------
+        int 0   no plot
+            1   plot
+            2   plot but dont show (for testing)
+        """
         if isinstance( plot, str ) :
-            return plot == 'iter' or plot == 'all'
+            if plot == 'iter' or plot == 'all' :
+                return 1
+            elif plot == 'test' :
+                return 2
+            else :
+                return 0
         else :
-            return False
+            return 0
 
     def doLastPlot( self, plot ) :
+        """
+        Return
+        ------
+        True when the last plot is requested (plot equals 'last' or 'all' or True)
+        False otherwise
+        """
         if isinstance( plot, str ) :
             return plot == 'last' or plot == 'all'
         else :
             return plot == True
 
 
-    def initReport( self, keep=None, plot=False ) :
-
-        self.plotData( plot=self.doIterPlot( plot ) )
+    def initReport( self, keep=None ) :
 
         if self.verbose >= 1 :
             fitIndex = self.walkers[0].fitIndex
@@ -553,7 +612,7 @@ class NestedSampler( object ):
 
         tail = 0
         if self.verbose > 1 :
-            while fitIndex[-tail-1] < 0 : 
+            while fitIndex is not None and fitIndex[-tail-1] < 0 and len( fitIndex ) > tail + 1 : 
                 tail += 1
 
 #            tail = self.distribution.nphypar
@@ -572,86 +631,99 @@ class NestedSampler( object ):
     def iterReport( self, kw, tail, plot=False ) :
 
         if self.verbose >= 3 or ( self.verbose >= 1 and
-                                  self.iteration % 100 == 0 ):
+                                  self.iteration % self.repiter == 0 ):
             if self.verbose == 1 :
-                if ( self.iteration / 100 ) % 50 == 49 :
+                if ( self.iteration / self.repiter ) % 50 == 49 :
                     nwln = "\n"
                 else :
                     nwln = ""
                 print( ">", end=nwln, flush=True )
             else :
-                pl = self.walkers[kw].allpars[self.walkers[kw].fitIndex]
-                np = len( pl )
-#                tail = self.distribution.nphypar
-                print( "%8d %#10.3g %8.1f %#10.3g %6d "%( self.iteration, self.logZ,
-                    self.info, self.lowLhood, np ), fmt( pl, max=4, tail=tail ) )
-#                print( "%-16.16s " % self.engines[1], end="" )
-#                self.engines[1].printReport()
-
+                self.printIterRep( kw, tail=tail, max=4 )
 
             self.plotResult( self.walkers[kw], self.iteration, plot=self.doIterPlot( plot ) )
 
+
+    def printIterRep( self, kw, parfmt="%s", tail=0, max=None, indent=0, end="\n" ) :
+
+        pl = self.walkers[kw].allpars
+        fi = self.walkers[kw].fitIndex
+        if fi is not None : 
+            pl = pl[fi]
+        np = len( pl )
+
+        print( "%8d %#10.3g %8.1f %#10.3g %6d "%( self.iteration, self.logZ,
+                self.info, self.lowLhood, np ), 
+                parfmt % fmt( pl, max=max, tail=tail, indent=indent ), end=end )
+
+        """
+        if self.iteration % 1000 == 0 :
+            print( fmt( pl, max=None ) )
+            npars = self.walkers[kw].problem.npars
+            pmn, pmx = self.phantoms.getParamMinmax( self.lowLhood, npars )
+            print( fmt( pmn, max=None ) )
+            print( fmt( pmx, max=None ) )
+            print( fmt( np ), fmt( self.phantoms.length( npars ) ), 
+                   fmt( self.logdZ, format="%10.3g" ) )
+            for ky in self.phantoms.phantoms :
+                print( fmt( ky ), fmt( len( self.phantoms.phantoms[ky] ) ) )
+        """
+
     def lastReport( self, kw, plot=False ) :
 
         if self.verbose > 0 :
             if self.verbose == 1 :
-                print( "\nIteration   logZ        H     LowL     npar    parameters" )
-            pl = self.walkers[kw].allpars[self.walkers[kw].fitIndex]
-            np = len( pl )
-            print( "%8d %#10.3g %8.1f %#10.3g %6d "%( self.iteration, self.logZ,
-                    self.info, self.lowLhood, np ) )
-            print( fmt( pl, max=None ) )
+                print( "\nIteration     logZ        H       LowL     npar" )
+
+            self.printIterRep( kw, max=None, indent=13, parfmt="\nParameters  %s" )
+
+
+#            pl = self.walkers[kw].allpars
+#            fi = self.walkers[kw].fitIndex
+#            if fi is not None : 
+#                pl = pl[fi]
+#            np = len( pl )
+#            print( "%8d %#10.3g %8.1f %#10.3g %6d "%( self.iteration, self.logZ,
+#                    self.info, self.lowLhood, np ) )
+#            print( "Parameters  ", fmt( pl, max=None, indent=13 ) )
 
         if self.verbose >= 1 :
             self.report()
 
         if self.doLastPlot( plot ) :
-#            Plotter.plotFit( self.problem.xdata, self.problem.ydata, yfit=self.yfit, residuals=True )
-            Plotter.plotSampleList( self.samples, self.problem.xdata, self.problem.ydata, 
+            self.plotLast()
+
+    def plotLast( self ) :
+        Plotter.plotSampleList( self.samples, self.problem.xdata, self.problem.ydata, 
                     figsize=[12,8], residuals=True )
 
     def getMaxIter( self ) :
         """
         Return the maximum number of iteration.
         """
-        return max( self.minimumIterations, self.end * self.ensemble * self.info / self.worst )
+        maxi = ( self.iteration if self.logdZ < self.tolerance else 
+                 self.end * self.ensemble * self.info / self.worst )
+        maxi = max( self.minimumIterations, maxi )
+        return maxi if self.maxIterations is None else min( maxi, self.maxIterations )
 
-    def getMaxIterMED( self ) :
+    def nextIteration( self ) :
         """
-        Return the maximum number of iteration in case of EvidenceProblem.
-        """
-        try :
-            self.csEvid += [math.exp( self.logZ )]
-        except :
-            self.csEvid = [math.exp( self.logZ )]
-
-        if self.iteration < self.minimumIterations :
-            return self.minimumIterations
-
-        kmed = 0
-        medlim = 0.5 * self.csEvid[-1]
-        for cs in self.csEvid :
-            if cs > medlim : break
-            kmed += 1
-
-        return self.end * kmed
-
-
-    def getMaxIterEP( self ) :
-        """
-        Return the maximum number of iteration in case of EvidenceProblem.
+        Return True when a next iteration is needed. False to stop
         """
         if self.iteration < self.minimumIterations :
-            return self.minimumIterations
-        if ( self.samples[-1].logW < self.samples[-100].logW and 
-             self.samples[-1].logW < self.logZ - 16 ) :
-            return self.iteration
-        else :
-            return self.iteration + 100  
+            return True
+        if self.logdZ < self.tolerance :
+            return False
+        if self.maxIterations is not None and self.iteration > self.maxIterations :
+            return False
+        if self.iteration > self.end * self.ensemble * self.info / self.worst :
+            return False
+        return True
+
+    def __str__( self ):
+        """ Return the name of this sampler.  """
+        return str( "NestedSampler" )
 
-#   det getScale( self, walker ) :
-#       np = walker.model.npchain
-#       return self.distribution.getScale( walker.model, params=walker.allpars[:np] )
 
 #  ===================================================================================
     def optionalRestart( self ):
@@ -691,7 +763,7 @@ class NestedSampler( object ):
         """
 
         for kw in range( worst ) :
-            logWeight = self.logWidth + self.walkers[kw].logL
+            logWeight = self.logWidth + self.walkers[kw].logL + self.walkers[kw].logPrior
 
             # update evidence, logZ
             logZnew = numpy.logaddexp( self.logZ, logWeight )
@@ -700,8 +772,6 @@ class NestedSampler( object ):
             self.info = ( math.exp( logWeight - logZnew ) * self.lowLhood +
                     math.exp( self.logZ - logZnew ) * ( self.info + self.logZ ) - logZnew )
 
-#            print( kw, fmt( self.walkers[kw].logL ), fmt( self.logWidth ), fmt( logWeight ), fmt( logZnew ), fmt( self.info ) )
-
             if math.isnan( self.info ) :
                 self.info = 0.0
             self.logZ = logZnew
@@ -710,10 +780,9 @@ class NestedSampler( object ):
             smpl = self.walkers[kw].toSample( logWeight )
             self.samples.add( smpl )
 
-#            self.logWidth -= 1.0 / self.ensemble
             self.logWidth -= 1.0 / ( self.ensemble - kw )
 
-#        self.logWidth -= worst / self.ensemble
+        self.logdZ = logWeight - self.logZ
 
         return
 
@@ -728,9 +797,60 @@ class NestedSampler( object ):
         """
         for k in range( worst ) :
             kcp = self.rng.randint( worst, self.ensemble )
-            self.walkers.copy( kcp, k )
-            setatt( self.walkers[k], "parent", kcp )
-            setatt( self.walkers[k], "start", self.iteration )
+            self.walkers.copy( kcp, k, start=self.iteration )
+#            setatt( self.walkers[k], "parent", kcp )
+#            setatt( self.walkers[k], "start", self.iteration )
+#            setatt( self.walkers[k], "step", 0 )
+
+    def copyWalkerFromPhantoms( self, worst ):
+        """
+        Kill worst walker( s ) in favour of one of the others
+
+        Parameters
+        ----------
+        worst : int
+            number of Walkers to copy
+        """
+        plen = self.phantoms.length()
+
+        for k in range( worst ) :
+            while True :
+                kcp = self.rng.randint( plen )
+                if self.phantoms.phantoms[kcp].logL > self.lowLhood :
+                    break
+            self.walkers.copy( kcp, k, wlist=self.phantoms.phantoms,
+                               start=self.iteration )
+#            setatt( self.walkers[k], "parent", kcp )
+#            setatt( self.walkers[k], "start", self.iteration )
+
+    def copyWalkerFromDynamicPhantoms( self, worst ):
+        """
+        Kill worst walker( s ) in favour of one of the others
+
+        Parameters
+        ----------
+        worst : int
+            number of Walkers to copy
+        """
+        plen = self.phantoms.length()
+        for k in range( worst ) :
+            while True :
+                kcp = self.rng.randint( plen )
+    
+                for pk in self.phantoms.phantoms.keys() :
+                    pklen = self.phantoms.length( np=pk )
+                    if kcp < pklen :
+                        break
+                    kcp -= pklen
+
+                if self.phantoms.phantoms[pk][kcp].logL > self.lowLhood :
+                    break
+
+            self.walkers.copy( kcp, k, wlist=self.phantoms.phantoms[pk],
+                               start=self.iteration )
+#            setatt( self.walkers[k], "parent", kcp )
+#            setatt( self.walkers[k], "start", self.iteration )
+
 
 
     def updateWalkers( self, explorer, worst ) :
@@ -759,6 +879,20 @@ class NestedSampler( object ):
             self.distribution.scale = value
         elif name == "power" and isinstance( self.distribution, ExponentialErrorDistribution ) :
             self.distribution.power = value
+        elif name == "copymode" and value == 1 :
+            if self.problem.isDynamic() :
+                self.copyWalker = self.copyWalkerFromDynamicPhantoms
+            else :
+                self.copyWalker = self.copyWalkerFromPhantoms
+        elif name == "bestUpdate" :
+
+            self.myFitter = None
+            if Tools.subclassof( value, BaseFitter ) :
+                object.__setattr__( self, name, True )
+                self.myFitter = value
+            else :
+                object.__setattr__( self, name, value )
+
         else :
             object.__setattr__( self, name, value )
             if name == "verbose" :
@@ -782,7 +916,7 @@ class NestedSampler( object ):
         elif name == "evidence" :
             return self.logZ / math.log( 10.0 )
         elif name == "logZprecision" :
-            return math.sqrt( self.info / self.ensemble )
+            return math.sqrt( max( self.info, 0 ) / self.ensemble )
         elif name == "precision" :
             return self.logZprecision / math.log( 10.0 )
         elif name == "information" :
@@ -805,7 +939,8 @@ class NestedSampler( object ):
 
     #  *********DISTRIBUTIONS***************************************************
 
-    def setProblem( self, name, model=None, xdata=None, ydata=None, weights=None ) :
+    def setProblem( self, name, model=None, xdata=None, ydata=None, weights=None,
+                    accuracy=None ) :
         """
         Set the problem for this run.
 
@@ -825,6 +960,8 @@ class NestedSampler( object ):
             dependent variable
         weights : array_like or None
             weights associated with ydata
+        accuracy : float or array_like or None
+            of the data
 
         """
         problemdict = {
@@ -842,6 +979,7 @@ class NestedSampler( object ):
             if xdata is not None : self.problem.xdata = numpy.asarray( xdata )
             if ydata is not None : self.problem.ydata = numpy.asarray( ydata )
             if weights is not None : self.problem.weights = numpy.asarray( weights )
+            if accuracy is not None : self.problem.setAccuracy( accuracy=accuracy )
             return
 
         if not isinstance( name, str ) :
@@ -850,9 +988,11 @@ class NestedSampler( object ):
         name = str.lower( name )
         try :
             myProblem = problemdict[name]
-            self.problem = myProblem( model, xdata=xdata, ydata=ydata, weights=weights )
+            self.problem = myProblem( model, xdata=xdata, ydata=ydata, weights=weights,
+                                      accuracy=accuracy )
         except :
-            raise ValueError( "Unknown problem name %s" % name )
+            raise 
+#            raise ValueError( "Unknown problem name %s" % name )
 
 
 
@@ -867,7 +1007,7 @@ class NestedSampler( object ):
             None    distribution is Problem dependent.
             string  name of distribution; one of
                     "gauss", "laplace", "poisson", "cauchy", "uniform",
-                    "exponential", "bernoulli"
+                    "exponential", "gauss2d", "model", "bernoulli"
             ErrorDistribution use this one
         limits : None or [low,high] or [[low],[high]]
             None    no limits implying fixed hyperparameters (scale,power,etc)
@@ -903,10 +1043,10 @@ class NestedSampler( object ):
             self.distribution = UniformErrorDistribution( scale=scale, limits=limits )
         elif name == "exponential" :
             self.distribution = ExponentialErrorDistribution( scale=scale, power=power, limits=limits )
+        elif name == "gauss2d" :
+            self.distribution = Gauss2dErrorDistribution( scale=scale, limits=limits )
         elif name == "model" :
             self.distribution = ModelDistribution( scale=scale, limits=limits )
-#            self.getMaxIter = self.getMaxIterEP
-#            self.minimumIterations = max( self.minimumIterations, 100 )
         elif name == "bernoulli" :
             self.distribution = BernoulliErrorDistribution()
         else :
@@ -925,34 +1065,34 @@ class NestedSampler( object ):
         enginedict : dictionary of { str : Engine }
             connecting names to the Engines
         """
+        ## the same for all engines. 
+        self.phantoms = PhantomCollection( dynamic=self.problem.isDynamic() )
 
         if enginedict is None :
             enginedict = {
                 "galilean" : GalileanEngine,
-                "chord" :    ChordEngine,
-                "birth" : 	 BirthEngine,
-                "death" : 	 DeathEngine,
-                "struct": 	 StructureEngine,
-                "gibbs" : 	 GibbsEngine,
-                "step"  : 	 StepEngine }
+                "chord" : ChordEngine,
+                "birth" : BirthEngine,
+                "death" : DeathEngine,
+                "struct": StructureEngine,
+                "gibbs" : GibbsEngine,
+                "random": RandomEngine,
+                "step"  : StepEngine }
 
         if engines is None :
             engines = self.problem.myEngines()
 
         self.engines = []
-        usePhantoms = self.step is not None
-
         if isinstance( engines, str ) :
             engines = [engines]
         for name in engines :
             if isinstance( name, Engine ) :
                 engine = name
-                engine.members = self.walkers
-                engine.errdis = self.distribution
-                engine.verbose = self.verbose
-                if usePhantoms :
-                    engine.setWalker = engine.setWalkerAdd2List
-                self.engines += [engine]
+                engine.walkers  = self.walkers
+                engine.errdis   = self.distribution
+                engine.phantoms = self.phantoms
+                engine.verbose  = self.verbose
+                self.engines   += [engine]
                 continue
 
             if not isinstance( name, str ) :
@@ -961,8 +1101,8 @@ class NestedSampler( object ):
             try :
                 Eng = enginedict[name]
                 seed = self.rng.randint( self.TWOP31 )
-                engine = Eng( self.walkers, self.distribution,
-                              seed=seed, verbose=self.verbose )
+                engine = Eng( self.walkers, self.distribution, seed=seed, 
+                            phantoms=self.phantoms, verbose=self.verbose )
             except :
                 raise ValueError( "Unknown Engine name : %10s" % name )
 
@@ -970,6 +1110,53 @@ class NestedSampler( object ):
 
 
     #  *********INITIALIZATION***************************************************
+    def setInitialEngine( self, ensemble, allpars, fitIndex, startdict=None ):
+        """
+        Initialize the walkers at random values of parameters and scale
+
+        Parameters
+        ----------
+        ensemble : int
+            length of the walkers list
+
+        TBC : remove allpars and fitIndex
+
+        allpars : array_like
+            array of (hyper)parameters
+        fitIndex : array_like
+            indices of allpars to be fitted
+
+        startdict : dictionary of { str : Engine }
+            connecting a name to a StartEngine
+        """
+        if startdict is None :
+            startdict = { "start" : StartEngine }
+
+        # Make the walkers list
+#        walker = Walker( 0, self.problem, numpy.asarray( allpars ), fitIndex )
+#        self.walkers = WalkerList( walker=walker, ensemble=ensemble )
+
+        if self.initialEngine is not None:
+            # decorate with proper information
+            self.initialEngine.walkers = self.walkers
+            self.initialEngine.errdis = self.distribution
+
+        else :
+            try :
+                name = self.problem.myStartEngine()
+                StartEng = startdict[name]
+                seed = self.rng.randint( self.TWOP31 )
+                self.initialEngine = StartEng( self.walkers, self.distribution,
+                        seed=seed )
+            except :
+                raise ValueError( "Unknown StartEngine name : %10s" % name )
+
+        # Calculate logL for all walkers.
+        self.distribution.lowLhood = -math.inf
+
+        # pass the PhantomCollection to the initial engine
+        self.initialEngine.phantoms = self.phantoms
+
     def initWalkers( self, ensemble, allpars, fitIndex, startdict=None ):
         """
         Initialize the walkers at random values of parameters and scale
@@ -985,29 +1172,12 @@ class NestedSampler( object ):
         startdict : dictionary of { str : Engine }
             connecting a name to a StartEngine
         """
-        if startdict is None :
-            startdict = { "start" : StartEngine }
-
         # Make the walkers list
-        self.walkers = WalkerList( self.problem, ensemble, allpars, fitIndex )
+        walker = Walker( 0, self.problem, numpy.asarray( allpars ), fitIndex )
+        self.walkers = WalkerList( walker=walker, ensemble=ensemble )
 
-        if self.initialEngine is not None:
-            # decorate with proper information
-            self.initialEngine.members = self.walkers
-            self.initialEngine.errdis = self.distribution
+        self.setInitialEngine( ensemble, allpars, fitIndex, startdict=startdict )
 
-        else :
-            try :
-                name = self.problem.myStartEngine()
-                StartEng = startdict[name]
-                seed = self.rng.randint( self.TWOP31 )
-                self.initialEngine = StartEng( self.walkers, self.distribution,
-                        seed=seed )
-            except :
-                raise ValueError( "Unknown StartEngine name : %10s" % name )
-
-        # Calculate logL for all walkers.
-        self.distribution.lowLhood = -math.inf
         for walker in self.walkers :
             self.initialEngine.execute( walker.id, -math.inf )
 
@@ -1015,22 +1185,8 @@ class NestedSampler( object ):
 #            print( w.id, w.problem.model.npars, len( w.problem.model.parameters), len( w.allpars) )
 #            w.check( nhyp=self.distribution.nphypar )
 
-    def plotData( self, plot=False ):
-        """
-        Initialize the plot with data.
 
-        Parameters
-        ----------
-        plot : bool
-            whether to plot.
-        """
-        if not plot :
-            return
-        plt.figure( 'iterplot' )
-        plt.plot( self.problem.xdata, self.problem.ydata, 'k.' )
-        plt.show( block=False )
-
-    def plotResult( self, walker, iter, plot=False ):
+    def plotResult( self, walker, iter, plot=0 ):
         """
         Plot the results for a walker.
 
@@ -1040,30 +1196,37 @@ class NestedSampler( object ):
             the walker to plot
         iter : int
             iteration number
-        plot : bool
-            whether to plot.
+        plot : int (one of (0,1,2)
+            0 immediate return, no action
+            1 plot
+            2 plot but dont show (for testing)
         """
-        if not plot :
+        if plot == 0 :
             return
 
         plt.figure( 'iterplot' )
+
         if self.line is not None :
             ax = plt.gca()
-            plt.pause( 0.02 )
-            ax.lines.remove( self.line )
+            if plot < 2 :
+                plt.pause( 0.02 )       ## updates and displays the plot before pause
+            ax.remove( )
             self.text.set_text( "Iteration %d" % iter )
-        else :
-            self.text = plt.title( "Iteration %d" % iter )
-            self.ymin, self.ymax = plt.ylim()
+
+        if self.ymin is None :
+            self.ymin = numpy.min( self.problem.ydata )
+            self.ymax = numpy.max( self.problem.ydata )
 
         param = walker.allpars
         model = walker.problem.model
         mock = model.result( self.problem.xdata, param )
+        plt.plot( self.problem.xdata, self.problem.ydata, 'k.' )
+        self.text = plt.title( "Iteration %d" % iter )
         self.line, = plt.plot( self.problem.xdata, mock, 'r-' )
         dmin = min( self.ymin, numpy.min( mock ) )
         dmax = max( self.ymax, numpy.max( mock ) )
-        plt.ylim( dmin, dmax )
-        plt.show( block=False )
+        dd = 0.05 * ( dmax - dmin ) 
+        plt.ylim( dmin - dd, dmax + dd )
 
 
     def report( self ):
@@ -1071,11 +1234,14 @@ class NestedSampler( object ):
         Final report on the run.
         """
 #        print( "Rate        %f" % self.rate )
-        print( "Engines              success     reject     failed       best      calls" )
+        print( "Engines              success     reject     failed", end="" )
+        if self.bestBoost :
+            print( "       best", end="" )
+        print( "      calls" )
 
         for engine in self.engines :
             print( "%-16.16s " % engine, end="" )
-            engine.printReport()
+            engine.printReport( best=self.bestBoost )
         print( "Calls to LogL     %10d" % self.distribution.ncalls, end="" )
         if self.distribution.nparts > 0 :
             print( "   to dLogL %10d" % self.distribution.nparts )
