@@ -1,13 +1,16 @@
 import numpy as numpy
 import re
 
+from .Tools import shortName as ToolsShortName
 from .Tools import setAttribute as setatt
+from .Tools import printclass as printclass
+
 from .Dynamic import Dynamic
 
 __author__ = "Do Kester"
-__year__ = 2020
+__year__ = 2024
 __license__ = "GPL3"
-__version__ = "2.6.1"
+__version__ = "3.2.1"
 __url__ = "https://www.bayesicfitting.nl"
 __status__ = "Perpetual Beta"
 
@@ -25,7 +28,7 @@ __status__ = "Perpetual Beta"
 #  *
 #  * The GPL3 license can be found at <http://www.gnu.org/licenses/>.
 #  *
-#  *    2018 - 2020 Do Kester
+#  *    2018 - 2024 Do Kester
 
 class Problem( object ):
     """
@@ -55,6 +58,8 @@ class Problem( object ):
         dependent variable (static)
     weights : array_like
         weights associated with ydata
+    varyy : float or ndarry of shape (ndata,)
+        Variance in the errors of the ydata
     npars : int
         number of parameters in the model of the problem
     partype : float | int
@@ -66,7 +71,8 @@ class Problem( object ):
     """
 
     #  *************************************************************************
-    def __init__( self, model=None, xdata=None, ydata=None, weights=None, copy=None ):
+    def __init__( self, model=None, xdata=None, ydata=None, weights=None, 
+                  accuracy=None, copy=None ):
         """
         Problem Constructor.
 
@@ -80,6 +86,9 @@ class Problem( object ):
             dependent variable
         weights : array_like or None
             weights associated with ydata
+        accuracy : float or ndarray of shape (ndata,)
+            accuracy scale for the datapoints
+            all the same or one for each data point
         copy : Problem
             to be copied
 
@@ -92,10 +101,14 @@ class Problem( object ):
             self.model = model
             self.weights = None if weights is None else numpy.asarray( weights )
             self.partype = float
+            self.setAccuracy( accuracy=accuracy )
         else :
             self.xdata = copy.xdata
             self.ydata = copy.ydata
             self.weights = copy.weights
+            self.varyy = copy.varyy
+            self.accuracy = copy.accuracy
+            self.hasAccuracy = copy.hasAccuracy
             if copy.model and ( copy.model.isDynamic() or copy.model.isModifiable() ) :
                 self.model = copy.model.copy()
             else :
@@ -103,7 +116,7 @@ class Problem( object ):
             self.partype = copy.partype
 
         if self.model is None or not hasattr( self.model, "cyclic" ) :
-            self.cyclicCorrection = self.cycor0
+            pass                                    # default is no correction
         elif isinstance( self.model.cyclic, dict ) :
             self.cyclicCorrection = self.cycor2
         else :
@@ -117,6 +130,20 @@ class Problem( object ):
         return Problem( copy=self )
 
 
+    def __setattr__( self, name, value ):
+        """
+        Set attributes.
+
+        """
+        setatt( self, name, value )
+        if name == "weights" :
+            try :
+                delattr( self, "sumweight" )
+            except :
+                pass
+        else :
+            setatt( self, name, value )
+
     def __getattr__( self, name ) :
         """
         Return value belonging to attribute with name.
@@ -128,7 +155,8 @@ class Problem( object ):
         """
         if name == 'npars' :
             return self.model.npars
-
+        elif name == 'varyy' :
+            return 0
         elif name == 'sumweight' :            # Return the sum over weight vector.
             if self.hasWeights() :
                 self.sumweight = numpy.sum( self.weights )
@@ -138,11 +166,27 @@ class Problem( object ):
         elif name == 'ndata' :
             self.ndata = len( self.ydata )        # number of data points/tuples
             return self.ndata
+        elif name == 'logScale' :
+            return 0.0
         else :
             raise AttributeError( "Unknown attribute " + name )
 
         return None
 
+    def setAccuracy( self, accuracy=None ) :
+        """
+        set the value for accuracy.
+
+        Paramaters
+        ----------
+        accuracy : float or array of NDATA floats or None
+            Either one value for all or one for each data point
+            When None the value is set to 0 (for easy computational reasons)
+        """
+        self.hasAccuracy = accuracy is not None
+        self.accuracy = accuracy if self.hasAccuracy else 0
+        self.varyy = self.accuracy * self.accuracy
+
     def hasWeights( self ):
         """ Return whether it has weights.  """
         return self.weights is not None
@@ -179,10 +223,15 @@ class Problem( object ):
         res = self.ydata - ( self.result( param ) if mockdata is None else mockdata )
         return self.cyclicCorrection( res )
 
-    def cycor0( self, res ):
-        """
-        Returns the residuals, unadultered
 
+    def cyclicCorrection( self, res ):
+        """
+        No correction.
+
+        Returns 
+        -------
+            the residuals, unadultered
+        
         Parameters
         ----------
         res : array_like
@@ -249,7 +298,7 @@ class Problem( object ):
 
 
 
-    def weightedResiduals( self, param, mockdata=None, extra=False ) :
+    def XXXweightedResiduals( self, param, mockdata=None, extra=False ) :
         """
         Returns the (weighted) residuals, calculated at the xdata.
 
@@ -277,7 +326,7 @@ class Problem( object ):
             return res if self.weights is None else res * self.weights
 
 
-    def weightedResSq( self, param, mockdata=None, extra=False ) :
+    def weightedResSq( self, allpars, mockdata=None, extra=False ) :
         """
         Returns the (weighted) squared residuals, calculated at the xdata.
 
@@ -285,7 +334,7 @@ class Problem( object ):
 
         Parameters
         ----------
-        param : array_like
+        allpars : array_like
             values for the parameters.
         mockdata : array_like
             model fit at xdata
@@ -293,13 +342,13 @@ class Problem( object ):
             true  : return ( wgt * res^2, wgt * res )
             false : return wgt * res^2
         """
-        res = self.residuals( param, mockdata=mockdata )
+        res = self.residuals( allpars[:self.npars], mockdata=mockdata )
 
         resw = res if self.weights is None else res * self.weights
         return ( resw * res, resw ) if extra else resw * res
 
     def isDynamic( self ) :
-        return self.model.isDynamic()
+        return self.model is not None and  self.model.isDynamic()
 
 
     def domain2Unit( self, dval, kpar ) :
@@ -332,20 +381,17 @@ class Problem( object ):
     #  *****TOSTRING***********************************************************
     def __str__( self ):
         """ Returns a string representation of the model.  """
-        return self.baseName( )
+        return self._toString()
 
-    def _toString( self, spaces ) :
-        return self.baseName()
+    def _toString( self, indent="" ) :
+        if hasattr( self, "model" ) and self.model is not None :
+            return "%s of %s" % ( self.baseName(), self.model.shortName() )
+        else :
+            return self.baseName()
 
-
-    def shortName( self ):
-        """
-        Return a short version the string representation: upto first non-letter.
-
-        """
-        m = re.match( "^[a-zA-Z_]*", self.baseName() )
-        return m.group(0)
+    def shortName( self ) :
+        return ToolsShortName( self._toString() )
 
     def baseName( self ) :
-        return "Problem of %s" % self.model.shortName()
+        return "Problem"
 
