@@ -6,13 +6,14 @@ import numbers
 import sys
 import trace
 import re
+import inspect
 
 from astropy.table import Table
 
 __author__ = "Do Kester"
-__year__ = 2021
+__year__ = 2024
 __license__ = "GPL3"
-__version__ = "2.8.0"
+__version__ = "3.2.2"
 __url__ = "https://www.bayesicfitting.nl"
 __status__ = "Perpetual Beta"
 
@@ -30,7 +31,7 @@ __status__ = "Perpetual Beta"
 #  *
 #  * The GPL3 license can be found at <http://www.gnu.org/licenses/>.
 #  *
-#  *    2016 - 2021 Do Kester
+#  *    2016 - 2024 Do Kester
 
 
 def getItem( ilist, k ) :
@@ -51,6 +52,36 @@ def getItem( ilist, k ) :
              ilist[-1] if k >= len( ilist ) else ilist[k] )
 
 
+def firstIndex( iterable, condition=lambda x: True ) :
+    """
+    Returns the index of first item in the `iterable` that
+    satisfies the `condition`.
+
+    If the condition is not given, it returns 0
+
+    Parameters
+    ----------
+    iterable : iterable
+        to find the first item in
+    condition : lambda function
+        the condition
+
+    Raises
+    ------
+    StopIteration: if no item satysfing the condition is found.
+
+    >>> firstIndex( (1,2,3), condition=lambda x: x % 2 == 0)
+    2
+    >>> firstIndex( range( 3, 100 ) )
+    3
+    >>> firstIndex( () )
+    Traceback (most recent call last):
+    ...
+    StopIteration
+    """
+    return next( k for k,x in enumerate( iterable ) if condition( x ) )
+
+
 def getColumnData( xdata, kcol ) :
     """
     Return the kcol-th column from xdata
@@ -76,6 +107,11 @@ def isBetween( xs, x, xe ) :
     """
     return ( xs <= x <= xe ) or ( xe <= x <= xs )
 
+def getKwargs( **kwargs ) :
+    """
+    Return kwargs as dictionary
+    """
+    return kwargs
 
 def setAttribute( obj, name, value, type=None, islist=False, isnone=False ) :
 
@@ -221,14 +257,14 @@ def makeNext( x, k ) :
     """
     try :
         _xnext = x[-1]
-    except :
+    except Exception :
         _xnext = x
     while True :
         try :
             _xnext = x[k]
             yield _xnext
             k += 1
-        except :
+        except Exception :
             yield _xnext
 
 
@@ -240,7 +276,7 @@ def length( x ) :
         return 0
     try :
         return len( x )
-    except :
+    except Exception :
         return 1
 
 def toArray( x, ndim=1, dtype=None ) :
@@ -259,7 +295,7 @@ def toArray( x, ndim=1, dtype=None ) :
     """
     if isinstance( x, Table ) :
         return x
-    return numpy.array( x, dtype=dtype, copy=False, ndmin=ndim )
+    return numpy.array( x, dtype=dtype, ndmin=min( ndim, 2 ) )
 
 def isList( item, cls ) :
     """
@@ -316,49 +352,93 @@ def decorate( src, des, copy=True ) :
         if copy :
             try :
                 value = value.copy()
-            except :
+            except Exception :
                 pass
         object.__setattr__( des, key, value )
 
 
-def printclass( cls, nitems=8 ) :
+def subclassof( sub, cls ) :
+    """
+    Determine if sub inherits from the class cls
+    
+    Parameters
+    ----------
+    sub : class object
+        supposed sub class
+    cls : class object
+        supposed parent class
+    """
+    if not inspect.isclass( sub ) : 
+        return False
+
+    while sub != cls and sub is not object :
+        tre = inspect.getclasstree( [sub], unique=True )   
+        sub = tre[0][0]
+
+    return sub is cls
+
+def printclass( cls, nitems=8, printId=False ) :
     """
     Print the attributes of a class.
     """
     numpy.set_printoptions( precision=3, threshold=10, edgeitems=4 )
 
+    ###  Import Model here to avoid circular imports ###
+    from .Model import Model
+    from .Problem import Problem
+
     print( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
-    print( cls, " at ", id( cls ) )
+    print( cls, " at ", id( cls ) ) if printId else  print( cls )
     print( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
     atr = vars( cls )
     ld = list( atr.keys() )
     ld.sort()
     for key in ld :
-        print( "%-16.16s"%key, end="" )
+        print( "%-15.15s "%key, end="" )
         val = atr[key]
         if isinstance( val, (list,numpy.ndarray) ) :
-            printlist( val )
+            printlist( val, nitems=nitems )
         elif isinstance( val, str ) :
             print( shortName( val ) )
+        elif isinstance( val, ( Model, Problem ) ) :
+            print( val._toString( indent="                " ) )
         elif key == "model" :
             print( val.shortName( ) )
+        elif inspect.ismethod( val ) :
+            print( "-->", val.__name__ )
         else :
-            print( val )
+            try :
+                valstr = val.__str__()
+                if valstr.startswith( '<BayesicFitting' ) :
+                    print( valstr.split()[0].split( '.' )[-1] )
+                else : 
+                    print( valstr )
+            except Exception :
+                print( val )
+    print( "+++++++++++++++++++++++++++++++++++++++++++++++++++++++" )
+            
 
 def printlist( val, nitems=8 ) :
     nv = length( val )
     if nv == 0 or isinstance( val[0], numbers.Number ) :
-        print( val, nv )
+        if nv <= nitems :
+            print( val )
+        else :
+            print( val, nv )
         return
 
+#    if val.dimensions > 1 :
+#        print( "array of shape", val.shape )
+#        return
+
     sep = "["
     for k in range( min( nv, nitems ) ) :
         try :
             print( "%s%s"%(sep, val[k].__name__ ), end="" )
-        except :
+        except Exception :
             print( "%s%s"%(sep, shortName( str( val[k] ) ) ), end="" )
         sep = " "
-    print( "%s"%("... ]" if nitems < nv else "]"), nv )
+    print( "%s" % ( ( "... ] %d" % nv ) if nitems < nv else "]" ) )
 
 def shortName( val ):
     """
@@ -438,12 +518,10 @@ def average( xx, weights=None, circular=None ) :
         sw = numpy.sum( weights )
         xw = xx * weights
         sx = numpy.sum( xw )
-        s2 = numpy.sum( xw * xx )
         averx = sx / sw
 
         rr = xx - averx
         stdvx = math.sqrt( numpy.average( rr * rr, weights=weights ) )
-#        stdvx = math.sqrt( s2 / sw - averx * averx )
 
     else :
         range = circular[1] - circular[0]
