@@ -35,7 +35,7 @@ __status__ = "Development"
 #  *
 #  *    2016 Do Kester
 
-class TestDynamicModel( unittest.TestCase ):
+class Test( unittest.TestCase ):
     """
     Test harness for Fitter class.
 
@@ -45,6 +45,7 @@ class TestDynamicModel( unittest.TestCase ):
     def __init__( self, testname ):
         super( ).__init__( testname )
         self.doplot = ( "DOPLOT" in os.environ and os.environ["DOPLOT"] == "1" )
+        self.dofull = ( "DOFULL" in os.environ and os.environ["DOFULL"] == "1" )
 
     def testDynamic( self ) :
         print( "  Test Dynamic" )
@@ -331,6 +332,9 @@ class TestDynamicModel( unittest.TestCase ):
         ns.distribution.setLimits( [0.01,100] )
         ns.distribution.constrain = self.constrainPos
         ns.verbose = 2
+        if not self.dofull :
+            ns.ensemble = 10
+
         evid = ns.sample( plot=self.doplot )
 
         sl = ns.samples
@@ -476,7 +480,8 @@ class TestDynamicModel( unittest.TestCase ):
         print( m.npchain, m.npbase, m._next.npbase, len( allpars ) )
         print( "====================" )
 
-        seng = StartEngine( sl, errdis )
+        phc = PhantomCollection( dynamic=True )
+        seng = StartEngine( sl, errdis, phantoms=phc )
         for k, s in enumerate( sl ) :
             print( "Walker  ", s.id, s.problem.npars, s.problem.model.npars, s.problem.model.npchain )
             seng.execute( k, 0.0 )
@@ -486,8 +491,15 @@ class TestDynamicModel( unittest.TestCase ):
         print( "logl  ", fmt( logl, max=None ) )
         print( "lowl  ", fmt( lowl ) )
 
-        beng = BirthEngine( sl, errdis )
-        deng = DeathEngine( sl, errdis )
+#        Tools.printclass( phc )
+        for k in phc.phantoms.keys() :
+            for w in phc.phantoms[k] :
+                print( fmt( k ), fmt( w.logL ) )
+                print( fmt( w.allpars, tail=3 ) )
+
+
+        beng = BirthEngine( sl, errdis, phantoms=phc )
+        deng = DeathEngine( sl, errdis, phantoms=phc )
         print( "start  np ", m.npchain )
         for k, s in enumerate( sl ) :
             Tools.printclass( s )
@@ -497,6 +509,12 @@ class TestDynamicModel( unittest.TestCase ):
             suc = deng.execute( k, lowl )
             print( "Death  ", suc, s.problem.npars, s.problem.model.npars, s.problem.model.npchain )
 
+        for k in phc.phantoms.keys() :
+            for w in phc.phantoms[k] :
+                print( fmt( k ), fmt( w.logL ) )
+                print( fmt( w.allpars, tail=3 ) )
+
+
 
     def testNestedSampler( self ) :
         print( "========================" )
@@ -514,13 +532,17 @@ class TestDynamicModel( unittest.TestCase ):
         lolim = [-10]
         hilim = [+10]
         pm.setLimits( lowLimits=lolim, highLimits=hilim )
-        Tools.printclass( pm )
+#        Tools.printclass( pm )
 
         engines = None
         ns = NestedSampler( x, pm, y, seed=2031967, engines=engines )
 
+        ns.verbose = 1
         ns.distribution.setLimits( [0.01, 100] )
 
+        if not self.dofull :
+            ns.ensemble = 10
+
         logE = ns.sample( plot=self.doplot )
 
 
@@ -552,6 +574,9 @@ class TestDynamicModel( unittest.TestCase ):
         ns.distribution.setLimits( [0.01, 100] )
 #        ns.distribution.scale = 0.02
 
+        if not self.dofull :
+            ns.ensemble = 10
+
         logE = ns.sample( plot=self.doplot )
         print( "scale  ", ns.scale )
 
@@ -579,12 +604,17 @@ class TestDynamicModel( unittest.TestCase ):
         ns = NestedSampler( x, pm, y, seed=2031967 )
         ns.distribution.setLimits( [0.01, 100] )
 #        ns.distribution.scale = 0.02
+        ns.verbose = 2
+        ns.bestBoost = True
+
+        if not self.dofull :
+            ns.ensemble = 10
 
         logE = ns.sample( plot=self.doplot )
         sl = ns.samples
-        print( "maxlik   ", fmt( sl.maxLikelihoodParameters, max=None ) )
-        print( "median   ", fmt( sl.medianParameters, max=None ) )
-        print( "modus    ", fmt( sl.modusParameters, max=None ) )
+        print( "maxlik   ", fmt( sl.maxLikelihoodParameters, max=None, indent=10 ) )
+        print( "median   ", fmt( sl.medianParameters, max=None, indent=10 ) )
+        print( "modus    ", fmt( sl.modusParameters, max=None, indent=10 ) )
         print( "scale    ", fmt( ns.scale ) )
 
 
