@@ -10,6 +10,7 @@ from astropy import units
 import math
 import sys
 import matplotlib.pyplot as plt
+from StdTests import stdModeltest
 
 from BayesicFitting import *
 from BayesicFitting import formatter as fmt
@@ -40,7 +41,7 @@ __status__ = "Perpetual Beta"
 #  *
 #  *  2017 - 2021  Do Kester
 
-class TestErrorDistribution( unittest.TestCase ):
+class Test( unittest.TestCase ):
     """
     Test harness for Fitter class.
 
@@ -66,124 +67,6 @@ class TestErrorDistribution( unittest.TestCase ):
         self.doplot = ( "DOPLOT" in os.environ and os.environ["DOPLOT"] == "1" )
         numpy.set_printoptions(formatter={'float': '{: 0.3f}'.format})
 
-    def test1( self ) :
-        print( "====test1============================" )
-        nn = 10000
-        x = numpy.linspace( 0, 1, nn, dtype=float )
-        ym = 0.0 * x
-        nf = 0.01
-
-        model = PolynomialModel( 0 )
-        model.parameters = 0.0
-        problem = ClassicProblem( model=model, xdata=x, ydata=ym )
-        fitIndex=[0]
-
-        numpy.random.seed( 2345 )
-        noise = numpy.random.randn( nn )
-
-        errdis = GaussErrorDistribution( )
-        allpars = [0.0, 1.0]
-        print( errdis )
-        for k in range( 5 ) :
-            noise = numpy.random.randn( nn )
-            y = ym + nf * noise
-            problem.ydata = y
-            print( fmt( k ), fmt( nf ), fmt( errdis.getScale( problem, allpars ) ) )
-            nf *= 10
-        if self.doplot:
-            self.ploterrdis( noise, errdis, problem, allpars )
-
-        nf = 0.01
-        errdis = LaplaceErrorDistribution( )
-        print( errdis )
-        for k in range( 5 ) :
-            noise = numpy.random.laplace( size=nn )
-            y = ym + nf * noise
-            problem.ydata = y
-            print( fmt( k ), fmt( nf ), fmt( errdis.getScale( problem, allpars ) ) )
-            nf *= 10
-        if self.doplot:
-            self.ploterrdis( noise, errdis, problem, allpars )
-
-        nf = 0.01
-        errdis = UniformErrorDistribution( )
-        print( errdis )
-        for k in range( 5 ) :
-            noise = 0.5 - numpy.random.rand( nn )
-            y = ym + nf * noise
-            problem.ydata = y
-            print( fmt( k ), fmt( nf ), fmt( errdis.getScale( problem, allpars ) ) )
-            nf *= 10
-        if self.doplot:
-            self.ploterrdis( noise, errdis, problem, allpars )
-
-        nf = 0.01
-        errdis = CauchyErrorDistribution ( )
-        cp = CauchyPrior()
-        print( errdis )
-        for k in range( 5 ) :
-            noise = numpy.random.rand( nn )
-            noise = cp.unit2Domain( noise )
-            y = ym + nf * noise
-            problem.ydata = y
-            print( fmt( k ), fmt( nf ), fmt( errdis.getScale( problem, allpars ) ) )
-            nf *= 10
-        if self.doplot :
-            self.ploterrdis( noise, errdis, problem, allpars )
-
-
-        nf = 0.01
-        errdis = ExponentialErrorDistribution( )
-        allpars = [0.0, 1.0, 1.0]
-        print( errdis, "  power=1" )
-        for k in range( 5 ) :
-            noise = numpy.random.laplace( size=nn )
-            y = ym + nf * noise
-            problem.ydata = y
-            print( fmt( k ), fmt( nf ), fmt( errdis.getScale( problem, allpars ) ) )
-            nf *= 10
-
-        nf = 0.01
-        errdis = ExponentialErrorDistribution( )
-        allpars = [0.0, 1.0, 2.0]
-        print( errdis, "  power=2"  )
-        for k in range( 5 ) :
-            noise = numpy.random.randn( nn )
-            y = ym + nf * noise
-            problem.ydata = y
-            print( fmt( k ), fmt( nf ), fmt( errdis.getScale( problem, allpars ) ) )
-            nf *= 10
-
-        nf = 0.01
-        power = 10
-        errdis = ExponentialErrorDistribution ( )
-        allpars = [0.0, 1.0, 10.0]
-        print( errdis, "  power=%d" % power  )
-        for k in range( 5 ) :
-            noise = 2 * numpy.random.rand( nn ) - 1.0
-            y = ym + nf * noise
-            problem.ydata = y
-            print( fmt( k ), fmt( nf ), fmt( errdis.getScale( problem, allpars ) ) )
-            nf *= 10
-
-        allpars = [0.0, 1.0, power]
-        if self.doplot :
-            self.ploterrdis( noise, errdis, problem, allpars )
-
-    def ploterrdis( self, noise, errdis, problem, allpars ) :
-        num_bins = 20
-#        plt.hist( noise, num_bins, normed=1, facecolor='g', alpha=0.5 )
-        plt.hist( noise, num_bins, facecolor='g', alpha=0.5 )
-        xx = numpy.linspace( -0.3, 0.3, 601, dtype=float )
-        lik = numpy.zeros_like( xx )
-        for k,p in enumerate( xx ) :
-            allpars[0] = p
-            lik[k] = errdis.logLikelihood( problem, allpars )
-        maxlik = numpy.max( lik )
-        plt.plot( xx, numpy.exp( lik - maxlik ), 'r-' )
-        plt.title( errdis.__str__() )
-        plt.show()
-
 
     def testGaussErrorDistribution( self ):
 
@@ -454,9 +337,14 @@ class TestErrorDistribution( unittest.TestCase ):
         poly = PolynomialModel( 1 )
         param = numpy.asarray( [1,10], dtype=float )
         ged = LaplaceErrorDistribution( )
-        ggd = ExponentialErrorDistribution( )
+        ggd = ExponentialErrorDistribution( limits=[[0.1,0.5],[10,3]] )
         self.assertTrue( ggd.acceptWeight() )
 
+        self.assertTrue( ggd.hyperpar[0].prior.lowLimit == 0.1 )
+        self.assertTrue( ggd.hyperpar[0].prior.highLimit == 10 )
+        self.assertTrue( ggd.hyperpar[1].prior.lowLimit == 0.5 )
+        self.assertTrue( ggd.hyperpar[1].prior.highLimit == 3 )
+
         #   data = { -11, -9, -5, -5, -1, 1, 1, 5, 5, 8, 11 }
         #   f( x ) = { -10, -8, -6, -4, -2, 0, 2, 4, 6, 8, 10 }
         #   f - d=     1   1  -1   1  -1 -1  1 -1  1  0  -1
@@ -487,9 +375,11 @@ class TestErrorDistribution( unittest.TestCase ):
         lpfi = numpy.asarray( [0,1,-1] )
         logL = ged.logLikelihood( problem, lppar )
         lggL = ggd.logLikelihood( problem, ggpar )
+        altL = ggd.logLikelihood_alt( problem, ggpar )
 
-        print( "lggL  = %8.3f   logL  = %8.3f" % ( lggL, logL ) )
+        print( "lggL  = %8.3f   logL  = %8.3f   alt  = %8.3f" % ( lggL, logL, altL ) )
         assertAAE( logL, lggL )
+        assertAAE( altL, lggL )
 
         lLdata = ggd.logLdata( problem, ggpar )
         logL0 = numpy.sum( lLdata )
@@ -507,8 +397,10 @@ class TestErrorDistribution( unittest.TestCase ):
         print( "numpart = ", nL )
 #        print( "partalt = ", ged.partialLogL_alt( problem, lppar, lpfi ) )
         print( "numpart = ", nG )
-        print( "partalt = ", ggd.partialLogL_alt( problem, ggpar, ggfi ) )
+        aG = ggd.partialLogL_alt( problem, ggpar, ggfi )
+        print( "partalt = ", aG )
 
+        assertAAE( dG, aG, 5 )
         assertAAE( dG[:3], dL, 5 )
         assertAAE( dG, nG, 5 )
         assertAAE( dL, nL, 5 )
@@ -562,6 +454,10 @@ class TestErrorDistribution( unittest.TestCase ):
         param = numpy.asarray( [0.9,10], dtype=float )
         ged = GaussErrorDistribution( )
         ggd = ExponentialErrorDistribution( )
+
+        print( str( ged ) )
+        print( str( ggd ) )
+
         self.assertTrue( ggd.acceptWeight() )
 
         #   data = { -11, -9, -5, -5, -1, 1, 1, 5, 5, 8, 11 }
@@ -692,11 +588,6 @@ class TestErrorDistribution( unittest.TestCase ):
         printclass( ed )
 
 
-
-
-
-
-
     def testPoissonErrorDistribution( self ):
         print( "====== Test Poisson Error Distribution ======================" )
         poly = PolynomialModel( 1 )
@@ -707,6 +598,8 @@ class TestErrorDistribution( unittest.TestCase ):
         problem = ClassicProblem( model=poly, xdata=self.x, ydata=data )
 
         ped = PoissonErrorDistribution( )
+        print( str( ped ) )
+
         self.assertFalse( ped.acceptWeight() )
 
         logL = ped.logLikelihood( problem, param )
@@ -744,6 +637,10 @@ class TestErrorDistribution( unittest.TestCase ):
         print( "numpart = ", nL )
         assertAAE( dL, nL, 5 )
 
+        aG = ped.partialLogL_alt( problem, param, fitIndex )
+        print( "partalt = ", aG )
+        assertAAE( dL, aG, 5 )
+
         scale = 1.0
         for i in range( 10 ) :
             param = numpy.asarray( [8+i,4], dtype=float )
@@ -757,7 +654,7 @@ class TestErrorDistribution( unittest.TestCase ):
     def testBernoulliErrorDistribution( self ):
         print( "====== Test Bernoulli Error Distribution ======================" )
         poly = LogisticModel( fixed={0:1} )
-        param = numpy.asarray( [0,1], dtype=float )
+        param = numpy.asarray( [0.5,1.0], dtype=float )
         data = numpy.asarray( [0,0,0,0,0,1,1,1,1,1,1], dtype=int )
         data = numpy.asarray( [1,1,1,1,1,1,0,0,0,0,0], dtype=int )
 
@@ -768,6 +665,8 @@ class TestErrorDistribution( unittest.TestCase ):
         problem = ClassicProblem( model=poly, xdata=self.x, ydata=data )
 
         ped = BernoulliErrorDistribution( )
+        print( str( ped ) )
+
         self.assertFalse( ped.acceptWeight() )
 
         logL = ped.logLikelihood( problem, param )
@@ -789,10 +688,10 @@ class TestErrorDistribution( unittest.TestCase ):
 
         fitIndex = [0,1]
         dL = ped.partialLogL( problem, param, fitIndex )
-#        aL = ped.partialLogL_alt( problem, param, fitIndex )
         nL = ped.numPartialLogL( problem, param, fitIndex )
+        aL = ped.partialLogL_alt( problem, param, fitIndex )
         print( "partial = ", dL )
-#        print( "altpart = ", aL )
+        print( "altpart = ", aL )
         print( "numpart = ", nL )
         assertAAE( dL, nL, 5 )
 
@@ -853,6 +752,10 @@ class TestErrorDistribution( unittest.TestCase ):
         print( "numpart = ", nL )
         assertAAE( dL, nL, 2 )
 
+        aG = ced.partialLogL_alt( problem, param, fi )
+        print( "partalt = ", aG )
+        assertAAE( dL, aG, 5 )
+
 
         for i in range( 11 ):
             param = numpy.asarray( [i-5,5,1], dtype=float )
@@ -870,6 +773,8 @@ class TestErrorDistribution( unittest.TestCase ):
         problem = ClassicProblem( model=poly, xdata=self.x, ydata=self.data )
 
         ced = LaplaceErrorDistribution( )
+        print( str( ced ) )
+
         self.assertTrue( ced.acceptWeight() )
 
         param = numpy.asarray( [1,10,1], dtype=float )
@@ -937,6 +842,8 @@ class TestErrorDistribution( unittest.TestCase ):
         problem = ClassicProblem( model=poly, xdata=self.x, ydata=self.data )
 
         ced = UniformErrorDistribution( )
+        print( str( ced ) )
+
         print( "x   ", self.x )
         print( "y   ", self.data )
         self.assertTrue( ced.acceptWeight() )
@@ -1010,6 +917,8 @@ class TestErrorDistribution( unittest.TestCase ):
         ed2 = UniformErrorDistribution( )
 
         ced = MixedErrorDistribution( ed1, ed2  )
+        print( str( ced ) )
+
         param = numpy.asarray( [0.0, 0.3, 5, 0.7 ], dtype=float )
         if self.doplot :
             for k in range( 5 ) :
