@@ -10,7 +10,7 @@ import warnings
 from StdTests import stdModeltest
 
 from BayesicFitting import *
-
+from BayesicFitting import formatter as fmt
 
 __author__ = "Do Kester"
 __year__ = 2017
@@ -35,7 +35,7 @@ __status__ = "Development"
 #  *
 #  *  2006 Do Kester
 
-class TestModels( unittest.TestCase ):
+class Test( unittest.TestCase ):
     """
     Test harness for Models
 
@@ -46,6 +46,35 @@ class TestModels( unittest.TestCase ):
         super( ).__init__( testname )
         self.doplot = ( "DOPLOT" in os.environ and os.environ["DOPLOT"] == "1" )
 
+    def test0( self ) :
+        model = SincModel()
+        param = [1,2,3,4]
+        self.assertRaises( ValueError, model.__setattr__, "parameters", param )
+        model.parameters = [1,2,3]
+        model.stdevs = [1,2,3]
+        cm = model.copy()
+        self.assertTrue( all( model.stdevs == cm.stdevs ) )
+
+        cm += PolynomialModel( 1 )
+
+        self.assertRaises( IndexError, cm.isolateModel, 2 )
+
+        sm = PolySurfaceModel( 2 )
+        cm = PolynomialModel( 2 )
+        self.assertRaises( ValueError, cm.appendModel, sm, 1 )
+
+        sm = HarmonicDynamicModel( 1 )
+        cm = SplinesDynamicModel( knots=[1,2,3] )
+        self.assertRaises( ValueError, cm.appendModel, sm, 1 )
+
+        self.assertRaises( IndexError, model.getPrior, 6 ) 
+        self.assertRaises( IndexError, model.getParameterName, 6 ) 
+        self.assertRaises( IndexError, model.getParameterUnit, 6 ) 
+
+        prior = UniformPrior( limits=[0,1] )
+        self.assertRaises( IndexError, model.setPrior, 6, prior ) 
+
+
     def testToString( self ) :
         model = PadeModel( 1, 2 )
         model += PolynomialModel( 0 )
@@ -55,8 +84,13 @@ class TestModels( unittest.TestCase ):
 
         model = PadeModel( 1, 2 )
         model += PolynomialModel( 1 )
-        print( model )
-        print( model.shortName() )
+        name = str( model )
+        print( name )
+        self.assertTrue( name[-1] == 'x' )
+        self.assertTrue( name[-5] == '5' )
+        sn = model.shortName()
+        print( sn )
+        self.assertTrue( sn[-1] == 'l' )
 
     def testArctanModel( self ):
         x  = numpy.asarray( [-1.0, -0.8, -0.6, -0.4, -0.2, 0.0, 0.2, 0.4, 0.6, 0.8, 1.0] )
@@ -82,7 +116,18 @@ class TestModels( unittest.TestCase ):
         p = []
         stdModeltest( m, p )
 
-        print( "******CONSTANT*******************" )
+        print( "******CONSTANT VALUE*******************" )
+        sm = SineModel()
+        m = ConstantModel( fixedModel=sm )
+        m.values = [2.0, 0.0, 0.5]
+        stdModeltest( m, p, silent=False )
+
+        print( "******CONSTANT MODEL*******************" )
+        tbl = numpy.linspace( 0, 1, 11 )
+        m = ConstantModel( table=tbl )
+        stdModeltest( m, p )
+
+        print( "******CONSTANT TABLE*******************" )
         m = ConstantModel( values = 5.5 )
         stdModeltest( m, p )
 
@@ -102,24 +147,6 @@ class TestModels( unittest.TestCase ):
 
         stdModeltest( m, p, plot=self.doplot )
 
-    """
-    def testEtalonVarModel( self ):
-        x  = numpy.asarray( [-1.0, -0.8, -0.6, -0.4, -0.2, 0.0, 0.2, 0.4, 0.6, 0.8, 1.0] )
-        print( "******ETALON VAR***********************" )
-        m = EtalonVarModel( )
-        p = numpy.asarray( [1.2, 0.6, 2.0, 0.2], dtype=float )
-
-        stdModeltest( m, p, plot=self.doplot )
-
-    def testEtalonVarModel2( self ):
-        x  = numpy.asarray( [-1.0, -0.8, -0.6, -0.4, -0.2, 0.0, 0.2, 0.4, 0.6, 0.8, 1.0] )
-        print( "******ETALON VAR 2***********************" )
-        pm = BasicSplinesModel( knots=[-1.0, 0.1, 1.0] )
-        m = EtalonVarModel( fixed={1:pm} )
-        p = numpy.asarray( [1.0, 5.5, 0.2, 0.03, 0.04, 0.05, 0.02, 0.01], dtype=float )
-
-        stdModeltest( m, p, plot=self.doplot )
-    """
     def testExpModel( self ):
         x  = numpy.asarray( [-1.0, -0.8, -0.6, -0.4, -0.2, 0.0, 0.2, 0.4, 0.6, 0.8, 1.0] )
         print( "******EXP**************************" )
@@ -182,6 +209,14 @@ class TestModels( unittest.TestCase ):
         x  = numpy.asarray( [-1.0, -0.8, -0.6, -0.4, -0.2, 0.0, 0.2, 0.4, 0.6, 0.8, 1.0] )
         print( "******GAUSS*************************" )
         m = GaussModel( )
+
+        p = numpy.asarray( [1.2,-0.2,-0.3], dtype=float )
+        print( fmt( p ) )
+        pc = m.correctParameters( p )
+        print( fmt( pc ) )
+        self.assertTrue( pc[2] == 0.3 )
+
+
         p = numpy.asarray( [1.2,-0.2,0.3], dtype=float )
 
         stdModeltest( m, p, plot=self.doplot )
@@ -197,6 +232,12 @@ class TestModels( unittest.TestCase ):
         gm.addModel( pm )
         par = numpy.asarray( [3,0.2,0.2,0.1,0.1], dtype=float )
 
+        par[2] = -0.2
+        print( par )
+        pc = gm.correctParameters( par )
+        print( pc )
+
+
         stdModeltest( gm, par, plot=self.doplot )
 
     def testHarmonicModel( self ):
