@@ -1,14 +1,13 @@
 # run with : python3 -m unittest TestPrior
-# or :       python3 -m unittest TestPrior.TestPrior.testUniformPrior
+# or :       python3 -m unittest TestPrior.Test.testUniformPrior
 
 import unittest
 import numpy as numpy
+from numpy.testing import assert_array_almost_equal as assertAAE
 import math
 
-from BayesicFitting import Prior, UniformPrior, JeffreysPrior, ExponentialPrior
-from BayesicFitting import LaplacePrior, CauchyPrior, GaussPrior
-from BayesicFitting import CircularUniformPrior
-
+from BayesicFitting import *
+from BayesicFitting import formatter as fmt
 
 __author__ = "Do Kester"
 __year__ = 2017
@@ -36,7 +35,7 @@ __status__ = "Development"
 #  *
 #  *  2006-2015 Do Kester (JAVA CODE)
 
-class TestPrior( unittest.TestCase ) :
+class Test( unittest.TestCase ) :
     """
     Test harness for Fitter class.
 
@@ -58,6 +57,7 @@ class TestPrior( unittest.TestCase ) :
 
         prior.setLimits( [0,5] )
         print( "lowlim %f  highlim %f range %f"%( prior.lowLimit, prior.highLimit, prior._urng ) )
+        self.assertTrue( prior._urng == prior.getIntegral() )
         print( prior )
 
         values = {0.0:0.0, 0.5:2.5, 1.0:5.0 }
@@ -91,24 +91,34 @@ class TestPrior( unittest.TestCase ) :
         print( "===== CircularUniform Prior Tests ==========================\n" )
 #        self.assertWarns( UserWarning, CircularUniformPrior )
         prior = CircularUniformPrior( )
-        print( prior )
+        printclass( prior )
         self.assertTrue( prior._lowDomain == -math.inf )
         self.assertTrue( prior._highDomain == +math.inf )
         self.assertFalse( prior.isBound() )
 
         prior.setLimits( [0,5] )
         print( "lowlim %f  highlim %f range %f"%( prior.lowLimit, prior.highLimit, prior._urng ) )
-        print( prior )
+        printclass( prior )
 
         values = {0.0:0.0, 0.5:2.5, 1.0:5.0 }
         self.stdTestPrior( prior, values=values, utest=False )
 
+        prior = UniformPrior( limits=[1,4], circular=True )
+        printclass( prior )
+        self.stdTestPrior( prior )
+
+        prior = UniformPrior( circular=4 )
+        printclass( prior )
+        self.stdTestPrior( prior )
+
+
+
         cp = prior.copy( )
         print( cp )
-        self.stdTestPrior( cp, values=values, utest=False )
+        self.stdTestPrior( cp )
 
         prior = UniformPrior( numpy.asarray( [6.,48.] ) )
-        print( prior )
+        printclass( prior )
         values = {0.0:6, 0.5:27, 1.0:48 }
         self.stdTestPrior( prior, values=values, utest=False )
 
@@ -159,6 +169,7 @@ class TestPrior( unittest.TestCase ) :
 
         self.assertTrue( prior._umin == math.log( 1.0 ) )
         self.assertTrue( prior._urng == math.log( 6.0 ) )
+        self.assertTrue( prior._urng == prior.getIntegral() )
 
         values = {0.0: 1.0, 0.5: 2.449489742783178, 1.0:6.0}
         self.stdTestPrior( prior, values=values )
@@ -179,17 +190,35 @@ class TestPrior( unittest.TestCase ) :
         print( "----- Standard test ----------------------------------------" )
 
         for ku in range( 11 ) :
-            u = 0.1 * ku
+            u = 1/3 + 1/30 * ku if prior.isCircular() else 0.1 * ku
             d = prior.unit2Domain( u )
-#            uu = u * prior._urng + prior._umin
-#            print( ku, u, uu, math.exp( uu ), d, prior.isOutOfLimits( math.exp( uu ) ) )
+
             v = prior.domain2Unit( d )
             f = prior.unit2Domain( v )
             print( "Unit %10.7f %10.7f  Domain %10.7f %10.7f"%( u, v, d, f ) )
             self.assertAlmostEqual( d, f )
-            if utest :
+            if utest and ku < 10 :
                 self.assertAlmostEqual( v, u )
 
+        u = numpy.arange( 10 ) * 0.1
+        if prior.isCircular() :
+            u = 1/3 * ( u + 1 )
+        d = prior.unit2Domain( u )
+        v = prior.domain2Unit( d )
+        f = prior.unit2Domain( v )
+        print( fmt( u, max=None ) )
+        print( fmt( v, max=None ) )
+        assertAAE( u, v )
+        print( fmt( d, max=None ) )
+        print( fmt( f, max=None ) )
+        assertAAE( d, f )
+
+        lr0 = numpy.log( prior.result( d ) )
+        lr1 = prior.logResult( d )
+        print( fmt( lr0, max=None ) )
+        print( fmt( lr1, max=None ) )
+        assertAAE( lr0, lr1 )
+
         # define a number of x values
         xx = [-10,-5, -1, 0, 1, 3, 6, 6.1, 48, math.inf]
         for x in xx :
@@ -235,12 +264,9 @@ class TestPrior( unittest.TestCase ) :
         prior = ExponentialPrior( )
         print( prior )
         self.assertTrue( prior.scale == 1.0 )
-        self.assertTrue( prior.zeroFraction == 0 )
         self.assertTrue( prior._lowDomain == 0 )
         self.assertTrue( prior._highDomain == +math.inf )
         self.assertTrue( prior.isBound() )
-        self.assertTrue( prior._uval == 0.0 )
-        self.assertTrue( prior._shift == 1.0 )
 
         self.stdTestPrior( prior )
         self.domainTest( prior )
@@ -259,7 +285,6 @@ class TestPrior( unittest.TestCase ) :
 
         maxdom = prior.MAXVAL * prior.scale
         self.assertTrue( prior.scale == 10.0 )
-        self.assertTrue( prior.zeroFraction == 0 )
         self.assertAlmostEqual( prior.unit2Domain( 1 -1.0 / 1024 ), 69.3147180559945 )
         self.assertAlmostEqual( prior.unit2Domain( 0.5 ), 6.93147180559945 )
         self.assertAlmostEqual( prior.unit2Domain( 0.0 ), 0 )
@@ -268,7 +293,6 @@ class TestPrior( unittest.TestCase ) :
         cp = prior.copy( )
         print( cp )
         self.assertTrue( cp.scale == 10.0 )
-        self.assertTrue( cp.zeroFraction == 0 )
         self.assertAlmostEqual( cp.unit2Domain( 1 -1.0 / 1024 ), 69.3147180559945 )
         self.assertAlmostEqual( cp.unit2Domain( 0.5 ), 6.93147180559945 )
         self.assertAlmostEqual( cp.unit2Domain( 0.0 ), 0 )
@@ -276,52 +300,15 @@ class TestPrior( unittest.TestCase ) :
 
         self.stdTestPrior( cp )
 
-        self.assertRaises( ValueError, prior.__setattr__, "zeroFraction", 1.3 )
-        self.assertRaises( ValueError, prior.__setattr__, "zeroFraction", -0.3 )
-
-
-        prior.zeroFraction = 0.5
-        print( prior )
-        self.assertTrue( prior.scale == 10.0 )
-        self.assertTrue( prior.zeroFraction == 0.5 )
-        self.assertTrue( prior._uval == 0.0 )
-        self.assertTrue( prior._shift == 0.5 )
-
-        self.stdTestPrior( prior, utest=False )
-
-        maxdom = prior.MAXVAL * prior.scale
-        self.assertAlmostEqual( prior.unit2Domain( 1 - 1.0 / 2048 ), 69.3147180559945 )
-        self.assertAlmostEqual( prior.unit2Domain( 0.75 ), 6.93147180559945 )
-        self.assertAlmostEqual( prior.unit2Domain( 0.5 ), 0 )
-        self.assertAlmostEqual( prior.unit2Domain( 0.25 ), 0 )
-        self.assertAlmostEqual( prior.unit2Domain( 0.0 ), 0 )
-        self.assertTrue( prior.unit2Domain( 1.0 ) == maxdom )
-        print( prior.domain2Unit( 0 ) )
-        uv = prior.domain2Unit( 0 )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(uv) ) != uv )
-        print( "partial 0.0 : ", prior.partialDomain2Unit( 0.0 ) )
-        print( "numpart 0.0 : ", prior.numPartialDomain2Unit( 0.0 ) )
-        print( "partial 0.1 : ", prior.partialDomain2Unit( 0.1 ) )
-        print( "partial  1. : ", prior.partialDomain2Unit( 1.0 ) )
-        print( "partial 10. : ", prior.partialDomain2Unit( 10. ) )
-        self.assertAlmostEqual( prior.partialDomain2Unit(0.0 ), 0.05 )
-        print( prior.partialDomain2Unit( 9 ) )
-        print( prior.numPartialDomain2Unit( 9 ) )
-        self.assertAlmostEqual( prior.partialDomain2Unit(9 ), prior.numPartialDomain2Unit( 9 ), 4 )
-        self.assertAlmostEqual( prior.partialDomain2Unit(1 ), prior.numPartialDomain2Unit( 1 ), 4 )
-
-        prior.scale = 100.0
-        self.assertTrue( prior.scale == 100.0 )
 
         cp = ExponentialPrior( prior=prior, scale=10 )
         print( cp )
         maxdom = cp.MAXVAL * cp.scale
-        self.assertTrue( cp.zeroFraction == 0.5 )
         self.assertTrue( cp.scale == 10 )
         print( cp.domain2Unit( 0 ) )
-        self.assertAlmostEqual( cp.unit2Domain( 1 - 1.0 / 2048 ), 69.3147180559945 )
-        self.assertAlmostEqual( cp.unit2Domain( 0.75 ), 6.93147180559945 )
-        self.assertAlmostEqual( cp.unit2Domain( 0.4 ), 0 )
+        self.assertAlmostEqual( cp.unit2Domain( 1 - 1.0 / 2048 ), 76.24618986 )
+        self.assertAlmostEqual( cp.unit2Domain( 0.75 ), 13.862943611 )
+        self.assertAlmostEqual( cp.unit2Domain( 0.4 ), 5.108256237 )
         self.assertTrue( cp.unit2Domain( 1.0 ) == maxdom )
 
         prior = ExponentialPrior( 2 )
@@ -333,28 +320,6 @@ class TestPrior( unittest.TestCase ) :
         self.assertAlmostEqual( prior.unit2Domain(prior.domain2Unit(50) ), 50.0, 4 )
         self.assertAlmostEqual( prior.unit2Domain(prior.domain2Unit(100) ), maxdom )
 
-        prior.zeroFraction = 0.3
-        print( prior )
-        self.assertTrue( prior.scale == 2.0 )
-        self.assertTrue( prior.zeroFraction == 0.3 )
-        self.assertTrue( prior._uval == 0.0 )
-        self.assertTrue( prior._shift == 0.7 )
-
-        self.stdTestPrior( prior, utest=False )
-        print( prior._uval )
-        print( prior.domain2Unit( 0 ) )
-        print( prior._uval )
-        uv = prior.domain2Unit( 0 )
-        self.assertTrue( uv == prior.domain2Unit( 0) )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(0.0) ) <= 0.3 )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(0.1) ) <= 0.3 )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(0.2) ) <= 0.3 )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(0.3) ) <= 0.3 )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(0.7) ) == 0.7 )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(0.8) ) == 0.8 )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(0.9) ) == 0.9 )
-        self.assertTrue( prior.domain2Unit(prior.unit2Domain(1.0) ) == 1.0 )
-        self.assertAlmostEqual( prior.unit2Domain(prior.domain2Unit(0.0) ), 0.0 )
 
     def testLaplacePrior( self ):
         print( "===== Laplace Prior Tests ==============================\n" )
@@ -547,17 +512,24 @@ class TestPrior( unittest.TestCase ) :
 
         maxdom = prior.center + prior.MAXVAL * prior.scale
         mindom = prior.center - prior.MAXVAL * prior.scale
+        print( "Domain from %f to %f" % (mindom, maxdom ) )
+        print( prior.unit2Domain( 1.0 ), prior.unit2Domain( 0.75 ) ) 
         self.assertTrue( prior.unit2Domain( 1.0 ) == maxdom )
-        self.assertAlmostEqual( prior.unit2Domain( 0.75 ), 0.47693627620446982 )
+#        self.assertAlmostEqual( prior.unit2Domain( 0.75 ), 0.4769362762044699 )
+
+        # value for the new (better version of GaussPrior
+        self.assertAlmostEqual( prior.unit2Domain( 0.75 ), 0.6744897501960818 )
         self.assertAlmostEqual( prior.unit2Domain( 0.5 ), 0 )
-        self.assertAlmostEqual( prior.unit2Domain( 0.25 ), -0.47693627620446982 )
+        self.assertAlmostEqual( prior.unit2Domain( 0.25 ), -0.6744897501960818 )
+#        self.assertAlmostEqual( prior.unit2Domain( 0.25 ), -0.4769362762044699 )
         self.assertTrue( prior.unit2Domain( 0.0 ) == mindom )
 
         prior.scale = 10
         print( prior )
         maxdom = prior.center + prior.MAXVAL * prior.scale
         self.assertTrue( prior.scale == 10 )
-        self.assertAlmostEqual( prior.unit2Domain( 0.25 ), -4.7693627620446982 )
+        self.assertAlmostEqual( prior.unit2Domain( 0.25 ), -6.744897501960818 )
+#        self.assertAlmostEqual( prior.unit2Domain( 0.25 ), -4.769362762044699 )
         self.assertAlmostEqual( prior.unit2Domain( 0.5 ), 0 )
         self.assertTrue( prior.unit2Domain( 1.0 ) == maxdom )
 
@@ -567,9 +539,11 @@ class TestPrior( unittest.TestCase ) :
         maxdom = cp.center + cp.MAXVAL * cp.scale
         mindom = cp.center - cp.MAXVAL * cp.scale
         self.assertTrue( cp.scale == 10.0 )
-        print( cp.unit2Domain( 0.25 ), 100 - 4.7693627620446982 )
+
+        print( cp.unit2Domain( 0.25 ), 100 - 6.744897501960818 )
         print( cp.unit2Domain( 0.5 ), 100 )
-        self.assertAlmostEqual( cp.unit2Domain( 0.25 ), 100 - 4.7693627620446982 )
+        self.assertAlmostEqual( cp.unit2Domain( 0.25 ), 100 - 6.744897501960818 )
+#        self.assertAlmostEqual( cp.unit2Domain( 0.25 ), 100 - 4.769362762044699 )
         self.assertAlmostEqual( cp.unit2Domain( 0.5 ), 100 )
         self.assertTrue( cp.unit2Domain( 1.0 ) == maxdom )
         self.assertTrue( cp.unit2Domain( 0.0 ) == mindom )
@@ -592,7 +566,11 @@ class TestPrior( unittest.TestCase ) :
         print( "numpart  1. : ", prior.numPartialDomain2Unit( 1.0 ) )
         print( "partial 10. : ", prior.partialDomain2Unit( 10. ) )
         print( "numpart 10. : ", prior.numPartialDomain2Unit( 10. ) )
-        self.assertAlmostEqual( prior.partialDomain2Unit(0.0 ), 0.5641895835477563 )
+        self.assertAlmostEqual( prior.partialDomain2Unit(0.0 ), 0.3989422804014327 )
+#        self.assertAlmostEqual( prior.partialDomain2Unit(0.0 ), 0.5641895835477563 )
+
+        # Value for new (better) GaussPrior.
+        # self.assertAlmostEqual( prior.partialDomain2Unit(0.0 ), 0.3989422804014327 )
         print( prior.partialDomain2Unit( 9 ) )
         print( prior.numPartialDomain2Unit( 9 ) )
         self.assertAlmostEqual( prior.partialDomain2Unit(-9 ), prior.numPartialDomain2Unit( -9 ), 4 )
